! Codes for "Life-Cycle and Intergenerational Effects of Child Care Reforms" published in Quantitative Economics
! copyright by Marc K. Chan and Kai Liu, 2017

INCLUDE 'link_fnl_shared.h'   ! Use IMSL 6.0 Fortran libraries interface
!include 'link_f90_dll_smp.h'  ! Use IMSL 5.0 Fortran libraries interface
program test
    use likeparam           ! load likelihood estimation global variables
    use platform            ! load platform-specific global variables
    USE SUBROUTINES         ! load main set of subroutines
    USE DATASTORE           ! load global variables
    USE uminf_INT           ! load multivariate optimization routine
    USE U4INF_INT           ! load optimization initialization routine
    use mkl_vsl_type        ! will define new streams in simulation
    use mkl_vsl             ! will define new streams in simulation
    use mkl95_lapack        ! potrf 
    USE bconf_INT           ! load multivariate optimization routine (with simple bounds)
    implicit none   
    
    REAL(KIND=8)    ::  fdjac(3500,70)=0.d0     ! initialize for large stderr computation (modified 20160823 to allow for larger sample)
    
    REAL(KIND=8)    ::  rparam(7), fscale=1.d0          ! optimization routine parameters
    real(kind=8), DIMENSION(paranummax)    ::  xguess, solution, parastd, xscale=1.d0    ! optimization routine parameters    
    REAL(KIND=8), DIMENSION(paranummax)    ::  solutionsim
    REAL(KIND=8), DIMENSION(paranummax)    ::  lbound=-9.d9, ubound=9.d9
    INTEGER    ::  mloop
    real(kind=8), DIMENSION(paranummax,paranummax) ::  covmat
    real(kind=8)    ::  solutionmat(paranummax,4)
    character(3)   ::  solutionmatstar(paranummax)
    INTEGER    ::  long=0, iostatarr(20)=0, obscount=0
    NAMELIST / instruction / parain, lbound, ubound
    
    open(1, file=trim(path)//'path.txt', status='old',iostat=iostat)     ! Read path.txt
    read(1,*) path
    write(*,*) 'Input root path (enter slash for default path (VS fortran project directory):'     ! Read root path from standard input or external file
    write(*,*) 'Currently the path is: ', path
    read(*,*) path
    write(*,*) 'You have entered: ', path
    open(1, file=trim(path)//'instruction.txt', status='old',iostat=iostat)     ! Read instruction.txt
    print *, 'iostat for opening instruction.txt = ', iostat    
    READ(1,NML=instruction)
    close(1)    
    !CALL checkparamvalue(parain)
    write (*,*) 'Values of simstatus, cdynprog, cjob, ctype, cdisc, cquad:'
    write (*,'(100(I1,1X))') parain%simstatus, parain%cdynprog, parain%cquad
    
    CALL CPU_TIME(tstart)          ! record start time
    CALL U4INF(iparam, rparam)     ! call optimization initialization routine and reset optimization parameters
    rparam(1)=200.e0*rparam(1)      ! scaled gradient tolerance
    rparam(2)=200.e0*rparam(2)      ! scaled step tolerance
    iparam(3)=100000                ! maximum number of iterations (default 100)
    iparam(4)=10000000              ! maximum number of function evaluations (default 400)
    iparam(5)=1000000               ! maximum number of gradient evaluations (default 400)
!    iparam(6)=1
    
    ! read model parameters from file 
    open(1, file=trim(path)//'parainit.txt', status='old',iostat=iostat)
    print *, 'iostat for opening parainit.txt = ', iostat    
    read(1,*) (xguess(iloop),iloop=1,paranummax)
    close(1)                   
    CALL remapparam(xguess,parain)  ! load xguess into parain and save total number of parameters as parain%paranum
    paranum=parain%paranum
    nthread=parain%nthread    
    
    ! read datasets
    open(0, file=trim(path)//'gdata0.txt', status='old',iostat=iostatarr(1))           ! group-level data
    open(51, file=trim(path)//'gdata0id.txt', status='old',iostat=iostatarr(15))        ! sample id data    
    open(1, file=trim(path)//'gdata1.txt', status='old',iostat=iostatarr(2))           ! choice data
    open(11,file=trim(path)//'gdata1a.txt', status='old',iostat=iostatarr(2))           ! choice set data    
    open(2, file=trim(path)//'gdata2.txt', status='old',iostat=iostatarr(3))           ! wage data
    open(8, file=trim(path)//'gdata8.txt', status='old',iostat=iostatarr(15))        ! lagged dep variable data    
    open(41, file=trim(path)//'gdata4afdc.txt', status='old',iostat=iostatarr(9))      ! child care variables
    open(43, file=trim(path)//'gdata4wage.txt', status='old',iostat=iostatarr(11))      ! wage equation variables
    open(46, file=trim(path)//'gdata4work.txt', status='old',iostat=iostatarr(14))      ! work preference variables
    open(47, file=trim(path)//'gdata4disc.txt', status='old',iostat=iostatarr(18))      ! pregnancy equation variables
    open(48, file=trim(path)//'gdata4educ.txt', status='old',iostat=iostatarr(19))      ! partner income variables    
    PRINT *,'IOSTAT SUM for reading data files =', SUM(iostatarr)
    IF (SUM(iostatarr)>0) THEN
        PRINT *, 'error reading data files'
        PAUSE
        STOP
    END IF
    obscount=0          ! Count number of observations read from data file      
    do iloop=1,parain%groupsize        
        read(0,'(3i3, 100f13.7)') gdatain(iloop)%gsize, gdatain(iloop)%dptype, gdatain(iloop)%tlstart, gdatain(iloop)%vart1, gdatain(iloop)%vart2, gdatain(iloop)%varinc
        IF (parain%cvarinc==1) gdatain(iloop)%varinc=gdatain(iloop)%varinc *parain%varincratio     
        read(51,'(i8)') gdatain(iloop)%id     
        do jloop=1,gdatain(iloop)%gsize
            obscount=obscount+1        
            read(1,'(i2)') gdatain(iloop)%datain(jloop)%choice
            read(11,'(3i2)') gdatain(iloop)%datain(jloop)%choicecase, gdatain(iloop)%datain(jloop)%care1, gdatain(iloop)%datain(jloop)%care2
            read(2,'(100f13.7)') gdatain(iloop)%datain(jloop)%wage
            read(8,'(100f13.7)') gdatain(iloop)%datain(jloop)%lwork, gdatain(iloop)%datain(jloop)%cwage,  gdatain(iloop)%datain(jloop)%lcare,  gdatain(iloop)%datain(jloop)%lpreg, &
             gdatain(iloop)%datain(jloop)%ckid, gdatain(iloop)%datain(jloop)%age1, gdatain(iloop)%datain(jloop)%age2, &
             gdatain(iloop)%datain(jloop)%lwage, gdatain(iloop)%datain(jloop)%lworkft, gdatain(iloop)%datain(jloop)%workexp_pt, gdatain(iloop)%datain(jloop)%workexp_ft
            read(41,'(100f13.7)') gdatain(iloop)%datain(jloop)%varafdc            
            read(43,'(100f13.7)') gdatain(iloop)%datain(jloop)%varwage
            read(46,'(100f13.7)') gdatain(iloop)%datain(jloop)%varwork            
            read(47,'(100f13.7)') gdatain(iloop)%datain(jloop)%varpreg
            read(48,'(100f13.7)') gdatain(iloop)%datain(jloop)%varinc
            IF (parain%cvarinc==1) gdatain(iloop)%datain(jloop)%varinc=gdatain(iloop)%datain(jloop)%varinc *parain%varincratio     
        end do
    end do    
               
    print *, 'Number of observations read = ', obscount
    close(0)
    close(1)
    close(11)
    
    close(2)
    close(3)    
    close(4)
    close(5)
    close(6)
    close(7)    
    close(8)
    close(41)
    close(42)
    close(43)
    close(44)
    close(45)
    close(46)
    close(51)    
    close(52)
    close(53)
    close(47)
    close(48)
    
    ! open file to record iterations of likelihood function (for likefcn only)
    open (1, file=trim(path)//'iterate.txt',status='replace')      
    write (1,*) 'Finished reading dataset and parameters. Number of model parameters =', paranum    
    write (*,*) 'Finished reading dataset and parameters. Number of model parameters =', paranum    
    write (1,*) 'Number of parallel threads =', nthread
    write (*,*) 'Number of parallel threads =', nthread
    
    call system_clock(int4)    
        
    if (parain%simstatus==3) then                      ! test program speed
        WRITE(*,*) 'Testing program speed. Number of likelihood evaluations =', parain%testspeed
        do iloop=1,parain%testspeed    
            print *,iloop        
            CALL likefcn(paranum, xguess(1:paranum), fscale)
        end do
    elseif (parain%simstatus==4) then                      ! test program speed
        WRITE(*,*) 'Export simulated income and posterior probabilities'
        parain%simcog=1
        if (parain%simcog==1) open (100, file=trim(path)//'simcog_inc.txt',status='replace')        
        if (parain%simcog==1) open (101, file=trim(path)//'simcog_type.txt',status='replace')        
        CALL likefcn(paranum, xguess(1:paranum), fscale)
        do iloop=1,parain%groupsize
        do kloop=1,gdatain(iloop)%gsize
            var1=SUM(gdatain(iloop)%posterior * parain%factorw)
            var2=SUM(gdatain(iloop)%posterior * parain%factorwork)
            var3=SUM(gdatain(iloop)%posterior * parain%factorp)
            write (100, '(i8, 1x, 2(i3, 1x), 100(f13.7,1x) )') gdatain(iloop)%id, kloop+1, gdatain(iloop)%datain(kloop)%choice, &
                gdatain(iloop)%datain(kloop)%wage, gdatain(iloop)%datain(kloop)%siminc, gdatain(iloop)%datain(kloop)%varinc     
        end do
        write (101, '(i8, 1x, 100(f13.7,1x) )') gdatain(iloop)%id, var1, var2, var3, (gdatain(iloop)%posterior(jloop), jloop=1,parain%typenum)
        end do
    elseif (parain%simstatus==1) then                 ! launch optimization
        IF (parain%optroutine==0) THEN
            CALL UMINF (LIKEFCN, X=SOLUTION(1:paranum), xguess=xguess(1:paranum), iparam=iparam, rparam=rparam, fscale=fscale, xscale=xscale(1:paranum))
        ELSE
            CALL BCONF (LIKEFCN, IBTYPE=0, XLB=LBOUND, XUB=UBOUND, X=SOLUTION(1:paranum), xguess=xguess(1:paranum), iparam=iparam, &
                rparam=rparam, fscale=fscale, xscale=xscale(1:paranum))
        END IF
        CALL std(solution(1:paranum),parastd(1:paranum), covmat(1:paranum,1:paranum), parain%groupsize, paranum, fdjac(1:parain%groupsize,1:paranum))            
        !CALL std(solution(1:paranum),parastd(1:paranum), covmat(1:paranum,1:paranum), parain%groupsize, paranum)            
    elseif (parain%simstatus==0) then            ! given xguess, compute likelihood value and standard deviation of model parameters
        CALL likefcn(paranum, xguess(1:paranum), fscale)
        solution=xguess          
        CALL std(solution(1:paranum),parastd(1:paranum), covmat(1:paranum,1:paranum), parain%groupsize, paranum, fdjac(1:parain%groupsize,1:paranum))       
        !CALL std(solution(1:paranum),parastd(1:paranum), covmat(1:paranum,1:paranum), parain%groupsize, paranum)            
    elseif (parain%simstatus==2) then            ! launch counterfactual simulation and write output    
        open (14, file=trim(path)//'simulate4.txt',status='replace')               
                
        IF (long==1) parain%discount=0.d0           ! ???
        CALL likefcn(paranum, xguess(1:paranum), fscale)       ! run likefcn once to remap parameters and launch dynprog (if instructed)               
        
        do jloop=4,4        
            iostat = vslnewstream( parain%sstream, VSL_BRNG_MT2203, 801 )       ! simulate type, wage and measurement error
            if (iostat /= vsl_error_ok) print *, 'error new stream in SSTREAM'
            iostat = vslnewstream( parain%sstream2, VSL_BRNG_MT2203, 802 )      ! simulate choice
            if (iostat /= vsl_error_ok) print *, 'error new stream in SSTREAM2'            
            iostat = vslnewstream( parain%sstream3, VSL_BRNG_MT2203, 803 )      ! SIMULATE parameters
            if (iostat /= vsl_error_ok) print *, 'error new stream in SSTREAM3'
            
            IF (parain%simci==1) THEN         ! allow parameters to be random in simulation
                call potrf(covmat(1:paranum,1:paranum),'U',iostat)   ! gaussianmv requires UPPER triangular matrix
                if (iostat /= 0) print *, 'error in cholesky factorization'                        
                ! simulate parameters and outcomes
                DO mloop=1,100                
                    iostat = vdrnggaussianmv( VSL_METHOD_DGAUSSIAN_BOXMULLER2, parain%sstream3, 1, solutionsim(1:paranum), paranum, VSL_MATRIX_STORAGE_FULL, &
                    solution(1:paranum), covmat(1:paranum,1:paranum) )
                    if (iostat /= vsl_error_ok) print *, 'error produce stream in randnum_norm'             
                    CALL remapparam(solutionsim(1:paranum),parain)
                    DO iloop=1,parain%groupsize                                               
                        CALL simulatetype(gdatain(iloop), parain, dpdatain(gdatain(iloop)%dptype,:), jloop, parain%simsize, 10+jloop )    ! ind (default=4), simsize (default=10), writenum i/o file number (10 to 14)
                    end do
                END DO
            ELSE
                if (parain%simcog==1) open (100, file=trim(path)//'simcog.txt',status='replace')
                DO iloop=1,parain%groupsize       
                    CALL simulatetype(gdatain(iloop), parain, dpdatain(gdatain(iloop)%dptype,:), jloop, parain%simsize, 10+jloop )    ! ind (default=4), simsize (default=10), writenum i/o file number (10 to 14)
                end do                        
            END IF            
        END DO          
        goto 999
    end if   
    close(1)
    close(10)
    close(11)
    close(12)
    close(13)
    close(14)            

    ! Define solution matrix (parameter, standard error, t-value, pvalue)
    do iloop=1,paranum
        solutionmat(iloop,1)=solution(iloop)
        solutionmat(iloop,2)=parastd(iloop)
        solutionmat(iloop,3)=solution(iloop)/parastd(iloop)
        solutionmat(iloop,4)=pvalue(solution(iloop)/parastd(iloop),2)
        solutionmatstar(iloop)=pvaluestar(solutionmat(iloop,4))
    end do
    
    call system_clock(int5)
    
    CALL CPU_TIME(tend)          ! record end time    
    timetotal=(tend-tstart)/DBLE(parain%nthread)
    
    ! Write likelihood value of each individual to file
    open(1,file=trim(path)//'likelihood.txt',status='replace')     
    do iloop=1,parain%groupsize
        write(1, '(i5, 1x, f13.7)') iloop, farr(iloop)
    end do
    close(1)
    
    ! Write solution to screen and files       
    open(1,file=trim(path)//'output.txt',status='replace')     
    write(1,*) 'Number of groups and observations: ', parain%groupsize, obscount
    write(*,*) 'Time of operation was ', INT(timetotal/60.d0), ' minutes', INT((timetotal/60.d0 - INT(timetotal/60.d0))*60.d0), ' seconds.'
    write(1,*) 'Time of operation was ', INT(timetotal/60.d0), ' minutes', INT((timetotal/60.d0 - INT(timetotal/60.d0))*60.d0), ' seconds.'
    print *, 'system clock time count', int4, int5, int5-int4
    
    IF (parain%simstatus==3) goto 999
    write(*,*) 'Number of iterations and likelihood = ', NITER, currentmin
    write(1,*) 'Number of iterations and likelihood = ', NITER, currentmin
    write(*,*) 'Parameter #, estimate, standard error, t-value, p-value:'    
    write(1,*) 'Parameter #, estimate, standard error, t-value, p-value:'    
    do iloop=1,paranum
        write (*, '( I3, 4(f9.3, 1x), A3 )') iloop, (solutionmat(iloop,jloop), jloop=1,4), solutionmatstar(iloop)    
        write (1, '( I3, 4(f9.3, 1x), A3 )') iloop, (solutionmat(iloop,jloop), jloop=1,4), solutionmatstar(iloop)
    end do
    
    CALL deltamethod(11,solution,covmat,parain%ctype)         ! print deltamethod output   
    write (1,*) 'Initial parameter vector:'
    write (1, '(100(5(f13.7, 1x), /))') (xguess(iloop),iloop=1,paranum)
    write (1,*) 'Solution vector in high precision:'
    write (1, '(100(5(f13.7, 1x), /))') (solutionmat(iloop,1),iloop=1,paranum)
    iostat = vslnewstream( parain%sstream, VSL_BRNG_MT2203, 801 )       ! initialize stream for perturbation
    write (1,*) 'Solution vector with 10 percent perturbation:'
    write (1, '(100(5(f13.7, 1x), /))') (solutionmat(iloop,1)*(1.d0+(randnum(parain%sstream,'U')-0.5d0)*0.2d0),iloop=1,paranum)    
    close(1)        
    
    open(1,file=trim(path)//'output_covmat.txt',status='replace')     
    do iloop=1,paranum
        write (1, '( 100(f13.7, 1x) )') (covmat(iloop,jloop),jloop=1,paranum)
    end do
    close(1)
        
    open(1,file=trim(path)//'output_list.txt',status='replace')     
    !write (1,*) 'low-precision parameter in list form:'
    !write (1, '(100(10(f8.2, 1x), /))') solutionmat(:,1)            ! print 10 numbers in a row
    !write (1,*)
    write (1,*) 'high-precision estimate in list form:'    
    write (1, '(100(10(f13.7, 1x), /))') (solutionmat(iloop,1),iloop=1,paranum)
    write (1,*) 'high-precision standard error in list form:'
    write (1, '(100(10(f13.7, 1x), /))') (solutionmat(iloop,2),iloop=1,paranum)
    write (1,*) 'high-precision t value in list form:'
    write (1, '(100(10(f13.7, 1x), /))') (solutionmat(iloop,3),iloop=1,paranum)
    write (1,*) 'high-precision p value in list form:'
    write (1, '(100(10(f13.7, 1x), /))') (solutionmat(iloop,4),iloop=1,paranum)
    close(1)

    open(1,file=trim(path)//'output_parain.txt',status='replace')     
    write (1,*) 'Root path: ', path
    write (1,*) 'Parain object = '
    write (1,NML=instruction)                                      ! print full list of parameters in NAMELIST format
    close(1)    
            
    999 CALL BEEPQQ (4000,300)    
    CONTAINS
        SUBROUTINE deltamethod(index,solution,covmat,ctype)
            IMPLICIT NONE
            INTEGER,INTENT(IN)    ::  index,ctype
            REAL(KIND=8),INTENT(IN)    ::  solution(:), covmat(:,:)
            REAL(KIND=8)   ::  mat(2,2),cov(2,2),g1,g2,s1,s2            
            REAL(KIND=8)    :: temp1, temp2
            write(1,*) 'Choice error:'
            write(1,'(3(f13.7,1x))') EXP(solution(1)), parastd(1)*EXP(solution(1))
            write(1,*) 'log wage equation stddev:'
            !write(1,'(3(f13.7,1x))') EXP(solution(6)), parastd(6)*EXP(solution(6))            
            write(1,'(3(f13.7,1x))') EXP(solution(7)), parastd(7)*EXP(solution(7))            
            write(1,*) 'discont factor stddev:'
            temp1 = blogit(4.d0+solution(16))
            temp2 = parastd(16)*EXP(-4.d0-solution(16)) * (blogit(4.d0+solution(16)))**2.d0
            write(1,'(3(f13.7,1x))') temp1, temp2
            write(1,'(3(f13.7,1x))') temp1**4.d0, 4*(temp1**3.d0)*temp2
            IF (ctype==1) THEN
                g1=exp(solution(index))/(1.d0+exp(solution(index))+exp(solution(index+1)))
                g2=exp(solution(index+1))/(1.d0+exp(solution(index))+exp(solution(index+1)))            
                mat(1,1)=g1*(1.d0-g1)
                mat(1,2)=-g1*g2
                mat(2,1)=-g1*g2
                mat(2,2)=g2*(1.d0-g2)            
                cov=transpose(mat)*covmat(index:index+1,index:index+1)*mat
                s1=sqrt(cov(1,1))
                s2=sqrt(cov(2,2))            
                write(1,*) 'type probabilities:'
                write(1,'(3(f13.7,1x))') g1, s1
                write(1,'(3(f13.7,1x))') g2, s2
                g1 = exp(solution(index))/(1.d0+exp(solution(index)))
                write(1,'(3(f13.7,1x))') g1, sqrt(covmat(index,index))*g1*(1.d0-g1)
            END IF
        END SUBROUTINE deltamethod
            
end program test
