! Codes for "Life-Cycle and Intergenerational Effects of Child Care Reforms" published in Quantitative Economics
! copyright by Marc K. Chan and Kai Liu, 2017
    
MODULE platform
    IMPLICIT NONE
    integer, parameter   ::  DBL = SELECTED_REAL_KIND(p=13,r=200)
    REAL(KIND=DBL),PARAMETER   ::  tol=1.d-4
    INTEGER    ::  iloop,jloop,kloop, iostat
    CHARACTER(50)  ::  file,file1,file2,file3,file4,file5
    REAL(KIND=DBL)  ::  error
    REAL(KIND=DBL)  ::  var1,var2,var3,var4,var5
    INTEGER        ::  int1,int2,int3,int4,int5
    REAL(KIND=DBL)  ::  tstart, tend, timetotal
    CHARACTER(50),PARAMETER  ::  fmt1="(100f8.2)"
    INTEGER    ::  iparam(7)    
    TYPE   ::  ghqobj
        REAL(KIND=8)    ::  euler=0.577215664901532860606512090082d0    
        INTEGER        ::  npointdyn=3
        REAL(KIND=8)    ::  abscidyn(3)=(/-1.22474487139d0, 0.d0, 1.22474487139d0 /)*SQRT(2.d0)
        REAL(KIND=8)    ::  weightdyn(3)=(/ 0.295408975151d0, 1.1816359006d0, 0.295408975151d0 /)/SQRT(3.1415926535897932384626433832795D0)           
        INTEGER        ::  npoint=3
        REAL(KIND=8)    ::  absci(3)=(/-1.22474487139d0, 0.d0, 1.22474487139d0 /)*SQRT(2.d0)
        REAL(KIND=8)    ::  weight(3)=(/ 0.295408975151d0, 1.1816359006d0, 0.295408975151d0 /)/SQRT(3.1415926535897932384626433832795D0)                   
!        INTEGER        ::  npoint=5               
!        REAL(KIND=8)    ::  absci(5)=(/-2.02018287046d0,	-0.958572464614d0,	0.d0,	0.958572464614d0,	2.02018287046d0 /)*SQRT(2.d0)
!        REAL(KIND=8)    ::  weight(5)= (/ 0.019953242059d0,	0.393619323152d0,	0.945308720483d0,	&
!                                0.393619323152d0,	0.019953242059d0 /)/SQRT(3.1415926535897932384626433832795D0)    
    END TYPE ghqobj        
    
    CONTAINS
    
    ! example xx=(/5.d0, 7.d0/), locate(xx,1,x,j) x=4,j=0; x=6,j=1; x=8,j=2
    PURE SUBROUTINE locate(xx,n,x,j) ! n is the size of xx minus 1
        implicit none
        integer,intent(in)    ::  n
        integer,intent(out)   ::  j
        real(kind=8),intent(in)   ::  x,xx(0:n)
        DO j=0,n
            IF (x .LT. xx(j)) EXIT
        END DO
    END SUBROUTINE locate     
    PURE SUBROUTINE locate2(xx,x,j) ! xx is array, x is input, j takes value from 0 to SIZE(xx)
        implicit none
        integer,intent(out)   ::  j
        real(kind=8),intent(in)   ::  x,xx(:)
        DO j=0,SIZE(xx)-1
            IF (x .LT. xx(j+1)) EXIT
        END DO
    END SUBROUTINE locate2     
    
    ! univariate function linear interpolation
    PURE FUNCTION interp(arrsize,xx,yy,x)
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  arrsize        
        REAL(KIND=8),INTENT(IN),DIMENSION(arrsize)  ::  xx,yy
        REAL(KIND=8),INTENT(IN)  ::  x
        REAL(KIND=8)  ::  interp
        INTEGER    ::  index
        CALL locate(xx,arrsize-1,x,index)
        IF ((index .GT. 0) .AND. (index .LT. arrsize)) THEN
            interp = yy(index) + (yy(index+1)-yy(index))/(xx(index+1)-xx(index))*(x-xx(index))                                  
        ELSE IF (index .EQ. 0) THEN
            interp = yy(1) + (yy(2)-yy(1))/(xx(2)-xx(1))*(x-xx(1))
        ELSE
            interp = yy(arrsize-1) + (yy(arrsize)-yy(arrsize-1))/(xx(arrsize)-xx(arrsize-1))*(x-xx(arrsize-1))
        END IF
    END FUNCTION interp                                
    
    ! "vector" linear interpolation
    PURE FUNCTION interpvec(arrsize,yy_size,xx,yy,x)
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  arrsize, yy_size      
        REAL(KIND=8),INTENT(IN),DIMENSION(arrsize)  ::  xx
        REAL(KIND=8),INTENT(IN),DIMENSION(yy_size,arrsize)  ::  yy
        REAL(KIND=8),INTENT(IN)  ::  x
        REAL(KIND=8),DIMENSION(yy_size)  ::  interpvec
        INTEGER    ::  index
        CALL locate(xx,arrsize-1,x,index)
        IF ((index .GT. 0) .AND. (index .LT. arrsize)) THEN
            interpvec = yy(:,index) + (yy(:,index+1)-yy(:,index))/(xx(index+1)-xx(index))*(x-xx(index))                                  
        ELSE IF (index .EQ. 0) THEN
            interpvec = yy(:,1) + (yy(:,2)-yy(:,1))/(xx(2)-xx(1))*(x-xx(1))
        ELSE
            interpvec = yy(:,arrsize-1) + (yy(:,arrsize)-yy(:,arrsize-1))/(xx(arrsize)-xx(arrsize-1))*(x-xx(arrsize-1))
        END IF
    END FUNCTION interpvec    
    
    REAL(KIND=8) FUNCTION ncdf(x)      ! cumulative normal distribution function - unstable in some ranges
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  x
        REAL(KIND=8)    ::  t, f
        REAL(KIND=8), PARAMETER    ::  p=0.2316419d0, b1=0.31938d0, b2=-0.35656d0, b3=1.78148d0, b4=-1.82125d0, b5=1.33027d0, nconstant=0.39894228d0
        t=1.d0/(1.d0+p*x)
        f=nconstant*EXP(-(x**2/2.d0))
        ncdf = 1.d0 - f*( b1*t + b2*t**2 + b3*t**3 + b4*t**4 + b5*t**5)
    END FUNCTION ncdf        
    REAL(KIND=8) FUNCTION pvalue(zscore,side)      ! computes pvalue (in percent)
        use anordf_int          ! computes normal cdf
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  zscore  ! supply positive or negative t-stat
        INTEGER,INTENT(IN)    ::  side        ! side=1 (one-sided test), other values (two-sided test, default)
        IF (side==1) THEN
            pvalue = 1.D0 - anordf( ABS(zscore) )*100.d0
        ELSE
            pvalue = (1.D0 - anordf( ABS(zscore) ))*2.D0*100.d0       
        END IF
    END FUNCTION pvalue  
    CHARACTER(3) FUNCTION pvaluestar(pvalue)
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  pvalue      ! pvalue in percent
        IF (pvalue<=1.d0) THEN
            pvaluestar='***'
        ELSEIF (pvalue >1.d0 .AND. pvalue <= 5.d0) THEN
            pvaluestar='** '
        ELSEIF (pvalue >5.d0 .AND. pvalue <= 10.d0) THEN
            pvaluestar='*  '
        ELSE
            pvaluestar='   '
        END IF
    END FUNCTION pvaluestar  
    ! linear interpolation of ev(x)    
    REAL(KIND=8) PURE FUNCTION interpev(x,evlow,evhigh)
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  x,evlow,evhigh
        INTEGER    ::  intx
        REAL(KIND=8)    ::  e
        intx = INT(x)
        e = x-intx
        interpev = (1.d0-e)*evlow + e*evhigh
    END FUNCTION interpev    
END MODULE platform