! Codes for "Life-Cycle and Intergenerational Effects of Child Care Reforms" published in Quantitative Economics
! copyright by Marc K. Chan and Kai Liu, 2017
    
! Stores parameters and generic variables
MODULE DATASTORE
    use mkl_vsl_type, only: vsl_stream_state       ! load random stream object type
    use platform, only: iloop, ghqobj                      ! load this variable to define arrays only
    IMPLICIT NONE    
    INTEGER     ::  farrindex1,farrindex2       ! initialize for large stderr computation
    CHARACTER(50)  ::  path=' '             ! root path for reading data and parameter files (default $(ProjectDir))    
    INTEGER,PARAMETER :: varworkdim=1, varafdcdim=3, varwagedim=1, typenum=5, varpregdim=1
    INTEGER,PARAMETER :: paranummax=60
    INTEGER            ::  paranum=0
    ! Number of choices in model, number of simulations (not used), number of individuals, maximum number of data/simulation periods
    INTEGER,PARAMETER :: mdim=12, sample=100, groupsize=3500, maxgsize=13+5, gsize=6   ! for longsim simulate long-run outcomes     (increased groupsize 20160823)
    INTEGER,PARAMETER  ::  maxhorizon=12+1+5, ghqpoint=3, ghqpointdyn=3 
    INTEGER    ::  TID, CHUNKSIZE=1, NTHREAD=4           ! thread id, sample size for each thread
    !$omp threadprivate (tid)                               ! declare thread id to be private to each thread (tid is global. need this line to make it private)
    INTEGER, EXTERNAL ::  OMP_GET_THREAD_NUM              ! function, obtain thread number in computer (platform specific)
    REAL(KIND=8)    ::  farr(groupsize), readin(110)        ! likelihood array, number of model parameters to read from file
    TYPE (VSL_STREAM_STATE), dimension(8) :: stream       ! define an array of random stream object    
    TYPE   ::  dataobj
        INTEGER    ::  choice                      ! choice (1 to mdim)
        INTEGER    ::  choicecase                  ! case number of choice set (0,1,2,3)
        INTEGER    ::  care1        
        INTEGER    ::  care2        
        REAL(KIND=8)    ::  wage    
        REAL(KIND=8)    ::  lwork
        REAL(KIND=8)    ::  lcare
        REAL(KIND=8)    ::  lpreg
        REAL(KIND=8)    ::  ckid
        REAL(KIND=8)    ::  cwage ! work experience        
        REAL(KIND=8)    ::  age1
        REAL(KIND=8)    ::  age2        
        REAL(KIND=8)    ::  varwork(varworkdim)     ! work preference equation
        REAL(KIND=8)    ::  varafdc(varafdcdim)     ! child care equation
        REAL(KIND=8)    ::  varwage(varwagedim)     ! wage equation
        REAL(KIND=8)    ::  varpreg(varpregdim)     ! pregnancy equation
        REAL(KIND=8)    ::  ftax(2)                 ! federal tax parameter        
        REAL(KIND=8)    ::  varinc=0.d0                  ! partner's actual after-tax income
        REAL(KIND=8)    ::  siminc=0.d0             ! individual's total income
        REAL(KIND=8)    ::  lwage=0.d0              ! lagged wage (for simcog only)        
        REAL(KIND=8)    ::  lworkft=0.d0            ! lagged ft work status        
        REAL(KIND=8)    ::  workexp_pt=0.d0
        REAL(KIND=8)    ::  workexp_ft=0.d0        
        INTEGER    ::  factor          ! store simulated factors (unobserved effects)     
    END TYPE dataobj
    
    TYPE   ::  gdataobj
        TYPE (dataobj) ::  datain(maxgsize)    ! source data (for likelihood computation)
        INTEGER    ::  gsize=gsize                   ! length of panel data
        INTEGER    ::  tlstart=1                 ! start wave of ev' (depend on cohort)
        INTEGER    ::  id                      ! Family ID (for i/o only)
        INTEGER    ::  dptype          ! observed type of individual in the dp problem       
        REAL(KIND=8)    ::  vart1=0.d0      ! covariate that affects type prob
        REAL(KIND=8)    ::  vart2=0.d0      ! covariate that affects type prob
        REAL(KIND=8)    ::  varinc=0.d0   ! partner's after-tax permanent income (for solving DP problem)        
        REAL(KIND=8),DIMENSION(typenum)    ::  posterior=0.d0  ! add posterior probs 20140602
    END TYPE gdataobj            
    TYPE (gdataobj) :: gdatain(groupsize)
    
    TYPE   ::  evobj
        REAL(KIND=8)    ::  ev(maxhorizon,typenum,3,3,1,0:3,8)=0.d0      ! ev  (time,type,varinc,lwork,lcare,ckid,cwage)   ! no work, pt work, and ft work
    END TYPE evobj
    TYPE   ::  evbaseobj
        REAL(KIND=8)    ::  evbase(typenum,3,3,1,ghqpointdyn,mdim)=0.d0     !  (type,varinc,lwork,1,wage,choice)        ! no work, pt work, and ft work
    END TYPE evbaseobj
    
    TYPE   ::  dpdataobj
        TYPE (dataobj)  ::  evdatain            ! source data (for dynamic programming only)       
        TYPE (evbaseobj)::  evbase(0:3,8,0:4,0:4)  ! nchild, workexp, age1, age2
        TYPE (evobj)    ::  ev(0:4,0:4)            ! age1, age2
        INTEGER    ::  tlhor=maxhorizon          ! length of time horizon
        REAL(KIND=8)    ::  varinc=0.d0 ! partner's after-tax permanent income (for solving DP problem)        
    END TYPE dpdataobj        
    TYPE (dpdataobj) :: dpdatain(1,2)           ! store dynamic programming data (individual types, pre-post reform)
    
    ! store default values of parameters (can be changed in instruction.txt)
    TYPE   ::  paraobj     
        REAL(KIND=8)    ::  errmean(12)=0.d0              ! choice mean
        REAL(KIND=8)    ::  errvar=1.d0                     ! choice variance                
        REAL(KIND=8)    ::  errvar2=1.d0                    ! choice variance (if no child care choice)        
        REAL(KIND=8)    ::  wstat(2)=(/ 0.d0, 1.d0 /)       ! wage mean and variance        
        REAL(KIND=8)    ::  pvarwork(varworkdim)=0.d0       ! work preference parameters
        REAL(KIND=8)    ::  pvarworkft(varworkdim)=0.d0     ! work preference full time parameters
        REAL(KIND=8)    ::  pvarafdc(varafdcdim)=0.d0       ! care preference parameters
        REAL(KIND=8)    ::  pvarwage(varwagedim)=0.d0       ! wage equation parameters
        REAL(KIND=8)    ::  pvarpreg(varpregdim)=0.d0       ! preg equation parameters
        REAL(KIND=8)    ::  plwork=0.d0 ! lag work
        REAL(KIND=8)    ::  plwage=0.d0 ! lag work in wage eqn 
        REAL(KIND=8)    ::  pcwage=0.d0 ! work exp
        REAL(KIND=8)    ::  pcwageqd=0.d0   ! work exp 2
        REAL(KIND=8)    ::  plcare=0.d0 ! lag care
        REAL(KIND=8)    ::  plpreg=0.d0 ! lag preg
        REAL(KIND=8)    ::  pckid=0.d0  ! kid
        REAL(KIND=8)    ::  pckidqd=0.d0! kid2   
        REAL(KIND=8)    ::  typeprob(typenum)=(/1.d0, (0.d0, iloop=2,typenum) /) ! type probability
!        REAL(KIND=8)    ::  typeprob(typenum)=(/1.d0 /)          ! type probability        
        REAL(KIND=8)    ::  factorw(typenum)=0.d0           ! factor load (wage)
        REAL(KIND=8)    ::  factorwork(typenum)=0.d0        ! factor load (work preference)
        REAL(KIND=8)    ::  factorc(typenum)=0.d0           ! factor load (care preference)        
        REAL(KIND=8)    ::  factorp(typenum)=0.d0           ! factor load (preg preference)        
        REAL(KIND=8)    ::  factork(typenum)=0.d0           ! factor load (kids preference)                
        REAL(KIND=8)    ::  discount=0.9d0                 ! discount factor
        REAL(KIND=8)    ::  quadratic=0.0d0 
        REAL(KIND=8)    ::  ppreg_k = 0.d0
        REAL(KIND=8)    ::  pckid2 = 0.d0
        REAL(KIND=8)    ::  phn = 0.d0
        REAL(KIND=8)    ::  phn2= 0.d0
        REAL(KIND=8)    ::  pcn = 0.d0
        REAL(KIND=8)    ::  ppn = 0.d0
        REAL(KIND=8)    ::  ppn2= 0.d0
        REAL(KIND=8)    ::  pcn2 = 0.d0
        REAL(KIND=8)    ::  factort0(typenum)=0.d0
        REAL(KIND=8)    ::  factort1(typenum)=0.d0
        REAL(KIND=8)    ::  factort2(typenum)=0.d0
        REAL(KIND=8)    ::  pinc=0.d0
        REAL(KIND=8)    ::  pyh=0.d0
        INTEGER         ::  cinc2=0     
        REAL(KIND=8)    ::  pyh2=0.d0        
        INTEGER         ::  cinc3=0     
        REAL(KIND=8)    ::  pyc=0.d0
        REAL(KIND=8)    ::  pyp=0.d0
        REAL(KIND=8)    ::  nthread=4                       ! number of threads for OPENMP parallel construct
        INTEGER        ::  simstatus=-1                    ! directive: 1 launch optimization, 0 obtain standard errors, 2 counterfactual simulation.
        INTEGER        ::  testspeed=100                  ! directive: number of likelihood evaluations when testing speed of program
        INTEGER   ::   writesim=1                          ! directive: write simulation output to external file     
        INTEGER    ::  lcftype=0                        ! directive: long-run simulation. default lcftype=0 (no long-run simulation)     
        INTEGER    ::  typecnt=0       ! store total number of individual types in data            
        INTEGER    ::  simupdate=1    ! default: update state variables in simulation
        INTEGER    ::  simci=0    ! default: assume parameter is fixed in simulation
        INTEGER    ::  simsize=10      ! number of simulations        
        INTEGER        ::  optroutine=0    ! choose optimization routine. 0 if uminf, otherwise bconf                
        INTEGER        ::  longsim=0       ! simulation: 1 if simulate full horizon        
        INTEGER        ::  simcog=0        ! simulate posteriors for cognitive production estimation        
        
        INTEGER        ::  cdynprog=-1                     ! directive (loaded in master program): 0 not launch dyn programming, 1 otherwise        
        INTEGER        ::  cdynprogtype=0        
        INTEGER        ::  dpsdep=-1                       ! directive: 0 DP no state dependence; 1 DP with state dependence        
        INTEGER        ::  ctype=-1                        ! directive (loaded in master program): 0 no multiple unobserved types in model, 1 otherwise
        INTEGER        ::  cbig=-1                         ! 0 small model with intercepts only, 1 otherwise
        INTEGER        ::  cbigwage=0                
        INTEGER        ::  ccare   = 1
        INTEGER        ::  ccareint = 0
        INTEGER        ::  cvar    = 0       
        INTEGER    ::  ccwage=1     
        INTEGER     ::  ccwageqd=1  
        INTEGER        ::  cquad=-1 
        INTEGER    ::  crra=0                
        INTEGER    ::  creform=0   ! =1 if compute dp twice, =0 compute dp once assuming reform only, =2 compute dp once assuming no reform
        INTEGER    ::  cleave=0    ! =1 if maternity leave
        INTEGER     :: cpreg=0      ! =1 if complex functional form in pregnancy decisions
        INTEGER     ::  cpreg2=0    
        INTEGER     ::  ctypevar=0  ! =1 if covariate affects type probability
        INTEGER     ::  cinc=0      ! =1 if add partner's income
        INTEGER     ::  cpregbig=0    ! =1 if add covariate in preg equation
        INTEGER     ::  cworkft=0    ! =1 if add covariate in ft work equation
        REAL(KIND=8)    ::  leaveratio = 1.d0
        REAL(KIND=8)    ::  fullcfc = 1.d0      ! =0 if only provide cfc benefits to nonworkers
        REAL(KIND=8)    ::  bonus = 0.d0        ! extra child subsidy
                
        INTEGER        ::  npwork=1, npafdc=1, npwage=1, nppreg=1, nptype=1   ! number of parameters estimated in model
        INTEGER        ::  paranum=-1
        INTEGER        ::  dirsim=0
        INTEGER    ::  mdim=mdim
        INTEGER    ::  sample=sample
        INTEGER    ::  groupsize=groupsize
        INTEGER    ::  maxgsize=maxgsize
        INTEGER    ::  gsize=gsize
        INTEGER    ::  typenum=typenum        
        INTEGER    ::  varworkdim=varworkdim
        INTEGER    ::  varafdcdim=varafdcdim
        INTEGER    ::  varwagedim=varwagedim
        INTEGER     ::  varpregdim=varpregdim
        TYPE(ghqobj)  ::   ghq 	         
        TYPE(vsl_stream_state) ::  sstream                 ! random stream
        TYPE(vsl_stream_state) ::  sstream2                ! random stream
        TYPE(vsl_stream_state) ::  sstream3                ! random stream
        INTEGER   ::   deltaindex=1                          ! delta method index for type probabilities
        INTEGER    ::  maxcwageloop=8        
        
        REAL(KIND=8)    ::  term_n1=0.d0
        REAL(KIND=8)    ::  term_n2=0.d0
        REAL(KIND=8)    ::  term_e1=0.d0
        REAL(KIND=8)    ::  term_e2=0.d0
        REAL(KIND=8)    ::  term_h=0.d0
        INTEGER         ::  cterm=0
        INTEGER         ::  npinc=1        
        INTEGER         ::  cplworkft=0
        REAL(KIND=8)    ::  plworkft=0.d0       ! lagged preference for ft work        
        INTEGER         ::  cftpreg=0
        REAL(KIND=8)    ::  ptexp_share=0.d0
        INTEGER         ::  cptexp=0        
        INTEGER         ::  cworkbig=1         
        INTEGER         ::  ccurrentreform=0           
        INTEGER         ::  cphn3=0
        REAL(KIND=8)    ::  phn3=0.d0
        INTEGER         ::  cerrvar2=0
        INTEGER         ::  cworkn0=0
        REAL(KIND=8)    ::  workn0=0.d0
        INTEGER     ::  cworkn1=0
        REAL(KIND=8)    ::  workn1=0.d0
        INTEGER     ::  cworkn2=0
        REAL(KIND=8)    ::  workn2=0.d0        
        REAL(KIND=8)    ::  leaveratio_pt=1.d0
        INTEGER         ::  ccohabit=0
        INTEGER         ::  cterm2=0
        INTEGER         ::  cleave_top=0
        REAL(KIND=8)    ::  mrate=0.d0          ! cohabit rate in simulation
        INTEGER         ::  cvarinc=0
        REAL(KIND=8)    ::  varincratio=1.d0        
    END TYPE paraobj
    TYPE (paraobj) ::  parain, paraoutstd, paraouttstat, paraoutpval
    
    TYPE   ::  lcobj
        REAL(KIND=8)    ::  wage
        INTEGER    ::  choice
        REAL(KIND=8)    ::  lincom(12)=0.d0
        REAL(KIND=8)    ::  wlincom=0.d0
        REAL(KIND=8)    ::  wstdlincom=0.d0
        REAL(KIND=8)    ::  lincomf(12)=0.d0
        REAL(KIND=8)    ::  wlincomf=0.d0
        REAL(KIND=8)    ::  wstdlincomf=0.d0        
        REAL(KIND=8)    ::  quadratic=0.d0
        REAL(KIND=8)    ::  quadraticf=0.d0
        INTEGER        ::  choicecase
    END TYPE lcobj
        
END MODULE DATASTORE

module likeparam                                           ! module for likefcn only
    use datastore, only: paranummax
    IMPLICIT NONE
    real    ::  currentmin=0.e0, currentminarg(paranummax)=0.e0
    integer    ::  niter=0    
end module likeparam

MODULE bensim
    USE platform, only: DBL,tol
    CONTAINS                                       
        REAL(KIND=DBL) PURE FUNCTION tftax(earn,arr, nkid, ncare)
        ! This function computes income tax (monthly level)     
        ! earn - gross earnings
        ! rate1 - rate at bracket 1; ded - standard deduction + total exemption           
            use datastore,only: parain
            IMPLICIT NONE            
            REAL(KIND=DBL),INTENT(IN) :: earn
            REAL(KIND=DBL),INTENT(IN),DIMENSION(:)    ::  arr
            INTEGER,INTENT(IN), OPTIONAL    :: nkid, ncare
            REAL(KIND=DBL)  ::  lev,benmax,rate1,rate2,ded, b1, b2
            REAL(KIND=DBL)  ::  netinc, surtax, ded1
            rate1=0.095d0
            rate2=0.137d0
            b1=248.d0
            b2=272.d0            
            ded1 = MIN(0.20D0*earn, 32.6d0) + 25.d0
            ded=0.d0            
            IF (present(nkid) .and. present(ncare)) then
                IF (nkid==1 .and. ncare .gt. 0) then
                    ded=25.d0
                elseif (nkid==2 .and. ncare .gt. 0) then
                    ded=30.d0
                end if
                ded = ded + parain%bonus * min(nkid,2)     ! tax deduction for children
            END IF                        
            netinc= MAX(earn - ded - ded1, 0.d0)
            tftax = 0.078d0*earn + 0.28d0*netinc   ! national insurance contribution and flat tax
            surtax=0.d0
            if ( (netinc .GE. b1) .AND. (netinc .LT. b2) ) THEN
                surtax=(netinc-b1)*rate1
            elseif ( (netinc .GE. b2) ) THEN
                surtax=(b2-b1)*rate1 + (netinc - b2)*rate2
            end if
            tftax=tftax+surtax
        END FUNCTION tftax            
END MODULE bensim

MODULE SUBROUTINES
    CONTAINS
    
    ! Load choice, wage, explanatory variables from data; multiply by parameters to obtain linear combinations
    SUBROUTINE complincom(dataobji, parain, lc, dp, ckid, age1, age2, sim, lpreg)
        use mkl95_blas    ! for gemv, DOT
        Use datastore, only: dataobj, paraobj, lcobj
        IMPLICIT NONE
        TYPE(dataobj),INTENT(IN)  ::  dataobji
        TYPE(paraobj),INTENT(IN)  ::  parain
        TYPE(lcobj), INTENT(OUT)  ::  lc
        INTEGER,INTENT(IN)       ::  dp        ! dp=0 (not dp problem); dp=1 (dp problem)
        REAL(KIND=8)              :: temp1, temp2, care1=0, care2=0
        INTEGER,INTENT(IN),OPTIONAL  ::  ckid,age1,age2,SIM, lpreg        
        INTEGER        ::  birth=0 ! if birth=1 then mother cannot choose work
        integer     ::  agediff, extra
        REAL(KIND=8)    ::  workexp
        lc%choice=dataobji%choice
        lc%wage=dataobji%wage    
        
        IF (dp==0 .AND. PRESENT(sim)==.false.) THEN
            lc%choicecase = dataobji%choicecase
            care1 = 0.d0
            care2 = 0.d0
            birth = dataobji%lpreg
        ELSE
            call gencase(ckid, age1, age2, lc%choicecase,care1,care2)
            birth = lpreg
        END IF        
                    
        lc%lincom = 0.d0        
        temp1 = parain%errmean(4)*care1   ! exogenous child care for old child
        temp2 = parain%errmean(5)*care2   ! exogenous child care for young child              
        IF (lc%choicecase==0) THEN                    ! no child care choice
            lc%lincom(1) = temp1 + temp2 
            lc%lincom(2) = lc%lincom(1) + parain%errmean(2) 
            lc%lincom(3) = lc%lincom(1) + parain%errmean(3) 
            lc%lincom(4) = -9.d9
            lc%lincom(5) = -9.d9
            lc%lincom(6) = -9.d9            
            lc%lincom(7) = lc%lincom(1) + parain%errmean(10)
            lc%lincom(8) = lc%lincom(2) + parain%errmean(10) + parain%errmean(9)
            lc%lincom(9) = lc%lincom(3) + parain%errmean(10) + parain%errmean(9) + parain%errmean(8)            
            lc%lincom(10) = -9.d9
            lc%lincom(11) = -9.d9
            lc%lincom(12) = -9.d9
        ELSEIF (lc%choicecase==1) THEN            ! multiple child care (old and young children)
            lc%lincom(1) = 0.d0
            lc%lincom(2) = parain%errmean(2) + parain%workn2
            lc%lincom(3) = parain%errmean(3) + parain%workn1
            lc%lincom(4) =                     parain%errmean(4)
            lc%lincom(5) = parain%errmean(2) + parain%errmean(4) + parain%errmean(6) + parain%workn2 
            lc%lincom(6) = parain%errmean(3) + parain%errmean(4) + parain%errmean(6) + parain%workn0 + parain%workn1
            lc%lincom(7) = lc%lincom(1) + parain%errmean(10)
            lc%lincom(8) = lc%lincom(2) + parain%errmean(10) + parain%errmean(9)
            lc%lincom(9) = lc%lincom(3) + parain%errmean(10) + parain%errmean(9) + parain%errmean(8)            
            lc%lincom(10) = lc%lincom(4) + parain%errmean(10) 
            lc%lincom(11) = lc%lincom(5) + parain%errmean(10)  + parain%errmean(9) 
            lc%lincom(12) = lc%lincom(6) + parain%errmean(10)  + parain%errmean(9)  + parain%errmean(8) 
        ELSEIF (lc%choicecase==2) THEN         ! old child care
            lc%lincom(1) = 0.d0                                 
            lc%lincom(2) = parain%errmean(2) + parain%workn2   
            lc%lincom(3) = parain%errmean(3) + parain%workn1   
            lc%lincom(4) =                     parain%errmean(4)
            lc%lincom(5) = parain%errmean(2) + parain%errmean(4) + parain%errmean(6) + parain%workn2 
            lc%lincom(6) = parain%errmean(3) + parain%errmean(4) + parain%errmean(6) + parain%workn0 + parain%workn1
            lc%lincom(7) = lc%lincom(1) + parain%errmean(10)
            lc%lincom(8) = lc%lincom(2) + parain%errmean(10)  + parain%errmean(9)
            lc%lincom(9) = lc%lincom(3) + parain%errmean(10)  + parain%errmean(9)  + parain%errmean(8)           
            lc%lincom(10) = lc%lincom(4) + parain%errmean(10) 
            lc%lincom(11) = lc%lincom(5) + parain%errmean(10)+ parain%errmean(9) 
            lc%lincom(12) = lc%lincom(6) + parain%errmean(10) + parain%errmean(9) + parain%errmean(8)
        ELSEIF (lc%choicecase==3) THEN         ! young child care
            lc%lincom(1) = 0.d0             
            lc%lincom(2) = parain%errmean(2) + parain%workn2    
            lc%lincom(3) = parain%errmean(3) + parain%workn1    
            lc%lincom(4) =                     parain%errmean(4)
            lc%lincom(5) = parain%errmean(2) + parain%errmean(4) + parain%errmean(6) + parain%workn2    
            lc%lincom(6) = parain%errmean(3) + parain%errmean(4) + parain%errmean(6)    + parain%workn0 + parain%workn1
            lc%lincom(7) = lc%lincom(1) + parain%errmean(10)
            lc%lincom(8) = lc%lincom(2) + parain%errmean(10) + parain%errmean(9)
            lc%lincom(9) = lc%lincom(3) + parain%errmean(10) + parain%errmean(9) + parain%errmean(8)
            lc%lincom(10) = lc%lincom(4) + parain%errmean(10) 
            lc%lincom(11) = lc%lincom(5) + parain%errmean(10) + parain%errmean(9) 
            lc%lincom(12) = lc%lincom(6) + parain%errmean(10) + parain%errmean(9) + parain%errmean(8) 
        else
            print *,'error in lincom'
            stop         
        END IF                        
        IF (parain%cleave==1 .and. birth==1) THEN ! maternity leave then limit choices
            lc%lincom( (/2,3,5,6,7,8,9,10,11,12/) ) = -9.d9     ! only choice 1 and 4 are available
        END IF
        IF (dp==0 .AND. PRESENT(sim)==.false.) THEN            ! NO PREGNANCY IF THREE KIDS ALREADY
            IF (INT(dataobji%ckid) == 3) lc%lincom( (/7,8,9,10,11,12/) ) = -9.d9
        ELSE
            IF (INT(ckid) == 3) lc%lincom( (/7,8,9,10,11,12/) ) = -9.d9
        END IF        
        lc%wlincom=parain%wstat(1)
        lc%wstdlincom = parain%wstat(2)        
        lc%quadratic = parain%quadratic                
        !!! Recursive definitions
        IF (parain%dpsdep==1 .AND. dp==0) THEN ! execute when there are lagged variables and when likelihood is computed
            lc%lincom( (/2,3,5,6/) ) = lc%lincom( (/2,3,5,6/) ) + parain%plwork*dataobji%lwork            
            lc%lincom( (/8,9,11,12/) ) = lc%lincom((/8,9,11,12/) ) + parain%plwork*dataobji%lwork
            lc%lincom( (/7,8,9,10,11,12/) ) = lc%lincom( (/7,8,9,10,11,12/) ) + parain%plworkft*dataobji%lwork
            workexp = dataobji%workexp_ft + (dataobji%workexp_pt)*parain%ptexp_share      ! compute units of work experience given data            
                lc%wlincom = lc%wlincom + parain%pcwage *(workexp-1.d0) &
                    + (workexp-1.d0)**2.d0 * parain%pcwageqd                             
            if (lc%choicecase==1 .or. lc%choicecase==2 .or. lc%choicecase==3) then
                lc%lincom( (/4,5,6,10,11,12/) ) = lc%lincom( (/4,5,6,10,11,12/) ) + parain%plcare*dataobji%lcare
            end if
            if (lc%choicecase==0) then
                lc%lincom( (/7,8,9/) ) = lc%lincom( (/7,8,9/) ) + parain%plpreg*dataobji%lpreg
            elseif (lc%choicecase==1 .or. lc%choicecase==2 .or. lc%choicecase==3) then
                lc%lincom( (/7,8,9,10,11,12/) ) = lc%lincom( (/7,8,9,10,11,12/) ) + parain%plpreg*dataobji%lpreg
            else
                print *,'error lincom 2'
                stop
            end if            
            lc%lincom( (/3,6,9,12/) ) = lc%lincom( (/3,6,9,12/) ) + parain%phn * dataobji%ckid 
            lc%lincom( (/2,5,8,11/) ) = lc%lincom( (/2,5,8,11/) ) + parain%phn2 * indic(INT(dataobji%ckid) .GT. 1)
            lc%lincom( (/7,10/) ) = lc%lincom( (/7,10/) ) + parain%phn3 * indic(INT(dataobji%ckid)==0)            
            lc%lincom( (/4,5,6,10,11,12/) ) = lc%lincom( (/4,5,6,10,11,12/) ) + parain%pcn * max(dataobji%ckid - 1.d0,0.d0) + &
                parain%pcn2 * indic(INT(dataobji%age1) ==3)
            lc%lincom( (/7,8,9,10,11,12/) ) = lc%lincom( (/7,8,9,10,11,12/) ) + parain%ppn * indic(dataobji%ckid==1.d0) + &
                parain%ppn2 * indic(dataobji%ckid==1.d0) * indic(INT(dataobji%age1)==1 .or. INT(dataobji%age1)==2)
        END IF
        
        IF (parain%cbig==1) THEN
            lc%lincom( (/2,3,5,6,8,9,11,12/) ) = lc%lincom( (/2,3,5,6,8,9,11,12/) ) + DOT(parain%pvarwork, dataobji%varwork) 
            IF (parain%cworkft==1) lc%lincom( (/3,6,9,12/) ) = lc%lincom( (/3,6,9,12/) ) + DOT(parain%pvarworkft, dataobji%varwork)              ! note: this is modified to FT work ONLY
            lc%wlincom = lc%wlincom + DOT(parain%pvarwage, dataobji%varwage)        
            lc%lincom( (/7,8,9,10,11,12/) ) = lc%lincom( (/7,8,9,10,11,12/) ) + DOT(parain%pvarpreg, dataobji%varpreg)             
        END IF       
        
        ! Initialize factor-specific vectors
        lc%lincomf=lc%lincom
        lc%wlincomf=lc%wlincom
        lc%wstdlincomf=lc%wstdlincom         
        lc%quadraticf = lc%quadratic        
    END SUBROUTINE complincom
    
    SUBROUTINE gencase(ckid, age1, age2, choicecase,care1,care2)
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  ckid, age1, age2
        INTEGER,INTENT(OUT)   ::  choicecase
        real(kind=8),intent(out)   ::  care1,care2
        IF (ckid==0) THEN
            choicecase=0
            care1=0
            care2=0
        ELSEIF (ckid==1 .or. (ckid == 2 .and. age2==0)) THEN
            IF (age1==0) THEN
                choicecase=0
                care1=0
                care2=0
            ELSEIF (age1==1 .or. age1==2 .or. age1==3) THEN
                choicecase=2
                care1=0
                care2=0
            ELSEIF (age1==4) THEN
                choicecase=0
                care1=0
                care2=0
            else
                print *,'error in care1 care2, 0'
                stop
            END IF
        ELSEIF (ckid == 2 .and. (age2==1 .or. age2==2 .or. age2==3)) THEN
            if (age1==1 .or. age1==2 .or. age1==3) THEN
                choicecase=1
                care1=0
                care2=0
            elseif (age1==4) then
                choicecase=3
                care1=0
                care2=0
            else
                print *,'error in care1 care2 1'
                stop
            end if
        elseif (ckid == 2 .and. age2==4) then
            choicecase=0
            care1=0
            care2=0
        elseif (ckid==3) then
            choicecase=0
            care1=0
            care2=0
        else                       
            print *,'error in care1 care2'
            stop                
        end if          
    END SUBROUTINE gencase 

    ! Load linear combinations; add factor loadings to obtain factor-adjusted linear combinations
    SUBROUTINE complincomf(lc, parain, iloop, ckid)
        Use datastore, only: paraobj, lcobj    
        IMPLICIT NONE
        TYPE(paraobj),INTENT(IN)  ::  parain
        TYPE(lcobj), INTENT(INOUT)  ::  lc
        INTEGER,INTENT(IN)    ::  iloop
        REAL(KIND=8),INTENT(IN),OPTIONAL  ::  ckid
        lc%lincomf( (/ 2,3,5,6 /) ) = lc%lincom ( (/ 2,3,5,6 /) ) + parain%factorwork(iloop)
        lc%lincomf( (/ 8,9,11,12 /) ) = lc%lincom ( (/ 8,9,11,12 /) ) + parain%factorwork(iloop)
        lc%wlincomf = lc%wlincom + parain%factorw(iloop)       
        if (lc%choicecase==1 .or. lc%choicecase==2 .or. lc%choicecase==3) then
            lc%lincomf( (/4,5,6,10,11,12/) ) = lc%lincomf( (/4,5,6,10,11,12/) ) + parain%factorc(iloop)
        end if
        if (lc%choicecase==0) then
            lc%lincomf( (/7,8,9/) ) = lc%lincomf( (/7,8,9/) ) + parain%factorp(iloop)
        elseif (lc%choicecase==1 .or. lc%choicecase==2 .or. lc%choicecase==3) then
            lc%lincomf( (/7,8,9,10,11,12/) ) = lc%lincomf( (/7,8,9,10,11,12/) ) + parain%factorp(iloop)
        else
            print *,'error lincom 3'
        end if            
    END SUBROUTINE complincomf                   
           
    SUBROUTINE complincomfdyn(parain, lc, itype, lpreg, lwork, lcare, ckid, cwage, age1, age2) ! for computing evbase
        Use datastore, only: paraobj, lcobj    
        IMPLICIT NONE
        TYPE(paraobj),INTENT(IN)  ::  parain
        TYPE(lcobj), INTENT(INOUT)  ::  lc
        INTEGER,INTENT(IN)    ::  itype, lwork, lcare, lpreg, cwage, ckid, age1, age2
        INTEGER     ::  extra, agediff
        lc%lincomf = lc%lincom
        lc%lincomf( (/ 2,3,5,6 /) ) = lc%lincom ( (/ 2,3,5,6 /) ) + parain%factorwork(itype) + parain%plwork*indic(lwork .GE. 2) 
        lc%lincomf( (/ 8,9,11,12 /) ) = lc%lincom ( (/ 8,9,11,12 /) ) + parain%factorwork(itype) + parain%plwork*indic(lwork .GE. 2)            
        lc%wlincomf = lc%wlincom + parain%factorw(itype) + parain%plwage * indic(lwork .GE. 2) + parain%pcwage  * DBLE(cwage-1) &
                + DBLE(cwage-1)**2.d0 * parain%pcwageqd                                       
        lc%lincomf( (/7,8,9,10,11,12/) ) = lc%lincomf( (/7,8,9,10,11,12/) ) + parain%plworkft*indic(lwork .GE. 2)           ! allow ft work as a state variable        
        if (lc%choicecase==1 .or. lc%choicecase==2 .or. lc%choicecase==3) then
            lc%lincomf( (/4,5,6,10,11,12/) ) = lc%lincomf( (/4,5,6,10,11,12/) ) + parain%plcare*indic(lcare==2) + parain%factorc(itype)
        end if
        if (lc%choicecase==0) then
            lc%lincomf( (/7,8,9/) ) = lc%lincomf( (/7,8,9/) ) + parain%plpreg*indic(lpreg==2) + parain%factorp(itype)
        elseif (lc%choicecase==1 .or. lc%choicecase==2 .or. lc%choicecase==3) then
            lc%lincomf( (/7,8,9,10,11,12/) ) = lc%lincomf( (/7,8,9,10,11,12/) ) + parain%plpreg*indic(lpreg==2) + parain%factorp(itype)
        else
            print *,'error lincom 3'
        end if            
        lc%lincomf( (/3,6,9,12/) ) = lc%lincomf( (/3,6,9,12/) ) + parain%phn * ckid
        lc%lincomf( (/2,5,8,11/) ) = lc%lincomf( (/2,5,8,11/) ) + parain%phn2 * indic(ckid .gt. 1)                        
        lc%lincomf( (/7,10/) ) = lc%lincomf( (/7,10/) ) + parain%phn3 * indic(ckid==0)                        
        lc%lincomf( (/4,5,6,10,11,12/) ) = lc%lincomf( (/4,5,6,10,11,12/) ) + parain%pcn * max(DBLE(ckid) - 1.d0,0.d0) + &
            parain%pcn2 * indic(age1 ==3)
        lc%lincomf( (/7,8,9,10,11,12/) ) = lc%lincomf( (/7,8,9,10,11,12/) ) + parain%ppn * indic(ckid==1) + &
            parain%ppn2 * indic(ckid==1) * indic(age1==1 .or. age1==2)
        lc%wstdlincomf = lc%wstdlincom
    END SUBROUTINE complincomfdyn            

    SUBROUTINE initerr
        use datastore, only: stream
        use mkl_vsl
        use mkl_vsl_type
        IMPLICIT NONE
        INTEGER    ::  iostat, iloop             
        do iloop=1,8
            iostat=vslnewstream( stream(iloop), VSL_BRNG_MT2203,  700+iloop )       ! set seed for random stream array
            if (iostat /= vsl_error_ok) print *, 'error new stream in INITERR'            
        end do
    END SUBROUTINE initerr    
    
    SUBROUTINE lnden(out, in, mean, recstd2, lgstd)
        IMPLICIT NONE
        REAL(KIND=8), PARAMETER    ::  constant=-0.918938533204673d0
        REAL(KIND=8),INTENT(OUT)   ::  out
        REAL(KIND=8),INTENT(IN)    ::  in, mean, recstd2, lgstd ! recstd2=1/sigma**2; lgstd=log(sigma)
        out=constant-lgstd-0.5d0*recstd2*(in-mean)**2
    END SUBROUTINE lnden       
    SUBROUTINE lndeneasy(out, in, mean, sigma)
        IMPLICIT NONE
        REAL(KIND=8),INTENT(OUT)   ::  out
        REAL(KIND=8),INTENT(IN)    ::  in, mean, sigma
        CALL lnden(out, in, mean, 1.d0/(sigma**2), LOG(sigma) )
    END SUBROUTINE lndeneasy    
    
    ! Compute income for each choice (given wage and data in period t)
    SUBROUTINE geninc(income,wage,wagetreat,dataobji,ben, benmat, ckid, age1, age2, sim, lpreg, lwork, reform)
        USE mkl95_blas      ! FUNCTION DOT
        USE bensim          ! LOAD BENEFIT FORMULAS
        USE datastore, only: dataobj, parain
        IMPLICIT NONE
        TYPE(dataobj),INTENT(IN)  ::  dataobji
        REAL(KIND=8),INTENT(IN)    ::  wage, wagetreat  ! wagetreat is defunct
        REAL(KIND=8),INTENT(OUT)   ::  income(:)           
        REAL(KIND=8)    ::  wage2, afdc1, afdc2, tax2, tax3, tax5          
        REAL(KIND=8)    ::  beitc1,beitc2    
        REAL(KIND=8),INTENT(OUT),OPTIONAL ::  ben(:)    ! output benefit vector
        REAL(KIND=8),INTENT(OUT),OPTIONAL    ::  benmat(:,:)    ! output benefit matrix
        INTEGER,INTENT(IN),OPTIONAL  ::  lwork  ! input lagged work (defunct)
        REAL(KIND=8)            ::  pc1, pc2, pc, cfc, care1, care2
        REAL(KIND=8)            ::  e1,e2
        real(kind=8)            ::  RATIO, ratio4, ratio5
        INTEGER,INTENT(IN),optional    ::  ckid,age1,age2,sim, lpreg   ! input state variables and sim status
        INTEGER,INTENT(IN),OPTIONAL  ::  reform ! =1 if CFC, =0 if no CFC
        INTEGER    ::  birth=0, birthwork=0
        INTEGER    ::  choicecase, age1now, age2now
        REAL(KIND=8)    :: ckidnow
        REAL(KIND=8)    ::  constant1, constant2
               
        IF(PRESENT(ben)) ben = 0.d0
        IF(PRESENT(benmat)) benmat=0.d0
        pc  = 28.325d0      ! cost of one child care
        pc2 = 45.202d0      ! cost of two child care
        cfc = 24.d0
        IF (PRESENT(reform)) THEN
            IF (reform==1) THEN
                cfc = 24.d0
            ELSE
                cfc = 0.d0
            END IF
        END IF
        care1=0.d0
        care2=0.d0
        IF(PRESENT(sim)) THEN          ! if dp problem or simulation problem then use generated state variables
            CALL gencase(ckid, age1, age2, choicecase, care1, care2)
            ckidnow=ckid
            age1now=age1
            age2now=age2
            birth=lpreg
            birthwork=lwork                 ! lwork = 0,1,2
        ELSE    ! otherwise use state variables from data
            choicecase=dataobji%choicecase
            ckidnow= dataobji%ckid
            age1now= dataobji%age1
            age2now= dataobji%age2
            birth  =dataobji%lpreg
            IF (dataobji%lworkft==1) then   ! lwork = 0,1,2
                birthwork = 2
            elseif (dataobji%lwork==1) THEN
                birthwork = 1
            else
                birthwork = 0
            end if
        END IF               

        IF (parain%lcftype==1) THEN
            CFC=0.D0
        END IF
        IF (parain%lcftype==2) THEN
            e1 = INDIC(age1now==1 .or. age1now==2)
            e2 = INDIC(age2now==1 .or. age2now==2)
        ELSE
            e1 = 1.d0
            e2 = 1.d0
        END IF
        IF (parain%lcftype==3) THEN
            ratio=1.d0
        else
            ratio=0.d0
        end if
        IF (parain%lcftype==4) THEN
            ratio4=0.d0
        else
            ratio4=1.d0
        end if
        IF (parain%lcftype==5) THEN
            ratio5=0.d0
        else
            ratio5=1.d0
        end if       
        
        income=0.d0  
        wage2=wage*2.d0           ! full-time gross earnings (work choices only)
        IF (choicecase==0) THEN   ! no child care choice
            income(1) = 0.d0 
            income(2) = wage - tftax(wage, dataobji%ftax, INT(ckidnow), 0)
            income(3) = wage2 - tftax(wage2, dataobji%ftax, INT(ckidnow), 0)
            income(4) = 0.d0
            income(5) = 0.d0
            income(6) = 0.d0
            income(7) = income(1)
            income(8) = income(2)
            income(9) = income(3)
            income(10) = 0.d0
            income(11) = 0.d0
            income(12) = 0.d0                 
        ELSEIF (choicecase==1) THEN  ! multiple child care
            constant1 = -parain%pvarafdc(2)*dataobji%varafdc(2)
            constant2 = -pc2 + parain%pvarafdc(1)*dataobji%varafdc(1) + parain%pvarafdc(3)*dataobji%varafdc(3)
            income(1) = 0.d0                                + cfc*2.d0                                  +constant1
            income(2) = wage  - tftax(wage, dataobji%ftax, int(ckidnow), 0)  + cfc*2.d0*parain%fullcfc  +constant1
            income(3) = wage2 - tftax(wage2, dataobji%ftax, int(ckidnow), 0) + cfc*2.d0*parain%fullcfc  +constant1
            income(4) = 0.d0                                                 +constant2
            income(5) = wage  - tftax(wage, dataobji%ftax, INT(ckidnow), 2)  +constant2
            income(6) = wage2 - tftax(wage2, dataobji%ftax, INT(ckidnow), 2) +constant2
            income(7) = income(1)
            income(8) = income(2)
            income(9) = income(3)
            income(10)= income(4)
            income(11)= income(5)
            income(12)= income(6)       
            IF(PRESENT(ben)) ben = (/constant1,constant1,constant1,constant2,constant2,constant2,constant1,constant1,constant1,constant2,constant2,constant2/)
        ELSEIF (choicecase==2) THEN  ! old child care
            constant1 = - parain%pvarafdc(2)*dataobji%varafdc(2)
            constant2 = -pc + parain%pvarafdc(1)*dataobji%varafdc(1) + parain%pvarafdc(3)*dataobji%varafdc(3)
            income(1) = 0.d0                                + cfc                                   +constant1
            income(2) = wage - tftax(wage, dataobji%ftax, int(ckidnow), 0)   + cfc*parain%fullcfc   +constant1
            income(3) = wage2 - tftax(wage2, dataobji%ftax, int(ckidnow), 0) + cfc*parain%fullcfc   +constant1
            income(4) = 0.d0                                                 +constant2
            income(5) = wage  - tftax(wage, dataobji%ftax, INT(ckidnow), 1)  +constant2
            income(6) = wage2 - tftax(wage2, dataobji%ftax, INT(ckidnow), 1) +constant2
            income(7) = income(1)
            income(8) = income(2)
            income(9) = income(3)
            income(10)= income(4)
            income(11)= income(5)
            income(12)= income(6)                
            IF(PRESENT(ben)) ben = (/constant1,constant1,constant1,constant2,constant2,constant2,constant1,constant1,constant1,constant2,constant2,constant2/)
        ELSEIF (choicecase==3) THEN  ! young child care
            constant1 = - parain%pvarafdc(2)*dataobji%varafdc(2)
            constant2 = - pc + parain%pvarafdc(1)*dataobji%varafdc(1) + parain%pvarafdc(3)*dataobji%varafdc(3)
            income(1) = 0.d0                                   + cfc                                 +constant1
            income(2) = wage - tftax(wage, dataobji%ftax, int(ckidnow), 0)      + cfc*parain%fullcfc +constant1
            income(3) = wage2 - tftax(wage2, dataobji%ftax, int(ckidnow), 0)    + cfc*parain%fullcfc +constant1
            income(4) = 0.d0                                                 +constant2
            income(5) = wage  - tftax(wage, dataobji%ftax, INT(ckidnow), 1)  +constant2
            income(6) = wage2 - tftax(wage2, dataobji%ftax, INT(ckidnow), 1) +constant2
            income(7) = income(1)
            income(8) = income(2)
            income(9) = income(3)
            income(10)= income(4)
            income(11)= income(5)
            income(12)= income(6)      
            IF(PRESENT(ben)) ben = (/constant1,constant1,constant1,constant2,constant2,constant2,constant1,constant1,constant1,constant2,constant2,constant2/)
        else
            print *,'error geninc'
            stop            
        END IF
        IF (parain%cleave==1 .and. birth==1) THEN ! eligible for maternity leave benefit if worked last period
            income ( (/2,3,5,6,7,8,9,10,11,12/) ) = 0.d0        ! only choice 1 and 4 available
            IF (birthwork==1) THEN     
                IF (parain%cleave_top==1) THEN
                    income( (/1,4/) ) = income( (/1,4/) ) + min(wage * 1.d0 * 0.8d0 * parain%leaveratio_pt,240.d0)    ! only choice 1 and 4 available
                ELSEIF (parain%cleave_top==0) THEN
                    income( (/1,4/) ) = income( (/1,4/) ) + wage * 1.d0 * 0.8d0 * parain%leaveratio_pt
                ELSE
                    income( (/1,4/) ) = income( (/1,4/) ) + min(wage,240.d0) * 0.8d0 * parain%leaveratio_pt - tftax(min(wage,240.d0) * 0.8d0 * parain%leaveratio_pt, dataobji%ftax, 0 , 0)
                END IF
            elseif (birthwork==2) THEN          ! FULL TIME WORK LAST PERIOD
                IF (parain%cleave_top==1) THEN
                    income( (/1,4/) ) = income( (/1,4/) ) + min(wage * 2.d0 * 0.8d0 * parain%leaveratio,240.d0)    ! only choice 1 and 4 available
                ELSEIF (parain%cleave_top==0) THEN
                    income( (/1,4/) ) = income( (/1,4/) ) + wage * 2.d0 * 0.8d0 * parain%leaveratio
                ELSE
                    income( (/1,4/) ) = income( (/1,4/) ) + min(wage*2.D0,240.d0) * 0.8d0 * parain%leaveratio - tftax(min(wage*2.D0,240.d0) * 0.8d0 * parain%leaveratio, dataobji%ftax, 0 , 0)
                end if
            END IF
        END IF
        income = income + ckidnow * 11.64d0    ! child subsidy
    END SUBROUTINE geninc            
             
    ! Remap parameters from array X to relevant elements in parain
    SUBROUTINE remapparam(X,parain)
        USE datastore, only: paraobj, paranum
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  X(:)
        TYPE(paraobj),INTENT(INOUT)   ::  parain        
        INTEGER    ::  index1,index2
        REAL(KIND=8)   ::  temp
        temp=0.d0
        parain%errvar=EXP(X(1))/1.28d0              ! EXP(X(1)) is sigma_c*pi/sqrt(6). Normalize to sigma_c in model for emax calculation
        ! X(2) phi_H; X(3) phi_3; X(4) phi_AF; X(5) phi_5; X(6) phi_F; X(7) phi_7; X(8) phi_8
        IF (parain%ccare==1) THEN
            parain%errmean(1)=0.d0
            parain%errmean(2)=X(2)  ! work preference
            parain%errmean(3)=X(3)  ! ft work preference
            parain%errmean(4)=X(4)  ! utility cost for child care 1
            parain%errmean(10)= X(5)    ! pregnancy cost
            parain%wstat(1)=X(6)
            parain%wstat(2)=EXP(X(7))
            index1=7
            IF (parain%ccareint==1) THEN
                parain%errmean(6)=X(8)
                parain%errmean(7)=X(9)
                index1=9
            END IF
            IF (parain%ccareint==2) THEN
                parain%errmean(6)=X(8)
                parain%errmean(7)=X(9)            
                parain%errmean(8)=X(10)
                parain%errmean(9)=X(11)
                parain%errmean(10)=X(12)    
                index1=12
            END IF
            IF (parain%ccareint==3) THEN
                parain%errmean(6) = X(8)
                index1=8
            END IF                                
            
            IF (parain%cvar==1) THEN
                parain%errvar2=EXP(X(10))/1.28d0
                index1=10
            ELSE
                parain%errvar2=parain%errvar
            END IF
        ELSEIF (parain%ccare==0) THEN
            parain%errmean(1)=0.d0
            parain%errmean(2)=X(2)  ! work preference
            parain%errmean(3)=X(3)  ! ft work preference
            parain%wstat(1)=X(4)
            parain%wstat(2)=EXP(X(5))
            index1=5
        ELSE
            parain%errmean(1)=0.d0
            parain%errmean(2)=X(2)  ! work preference
            parain%errmean(3)=parain%errmean(2)
            parain%wstat(1)=X(3)
            parain%wstat(2)=EXP(X(4))
            index1=4
        END IF        
        
        IF (parain%dpsdep==1) THEN
            CALL remapsc(index1,X,parain%plwork)         
            IF (parain%ccwage==1) THEN 
                CALL remapsc(index1,X,parain%pcwage)
                IF (parain%ccwageqd==1) CALL remapsc(index1,X,parain%pcwageqd)
            END IF
        CALL remapsc(index1,X,parain%errmean(9))
        END IF
        
        IF (parain%ctype==1) THEN      ! only applicable if at least 2 types            
            call remap(index1,parain%nptype-1,X,parain%factort0(2:parain%typenum))  ! type-specific constant
            CALL remap(index1,parain%nptype-1,X,parain%factorwork(2:parain%typenum))
            CALL remap(index1,parain%nptype-1,X,parain%factorw(2:parain%typenum))             
            CALL remap(index1,parain%nptype-1,X,parain%factorp(2:parain%typenum))            
        END IF        
        IF (parain%ctype==0) parain%nptype=1        ! reset nptype to 1
        
        IF (parain%cpreg==1) THEN
            CALL remapsc(index1,X,parain%phn)            
            CALL remapsc(index1,X,parain%pcn)            
            CALL remapsc(index1,X,parain%ppn)        
            CALL remapsc(index1,X,parain%ppn2)                         
            CALL remapsc(index1,X,parain%pcn2)                   
        END IF

        IF (parain%cbig==1) THEN
            IF (parain%cworkbig==1) CALL remap(index1,parain%npwork,X,parain%pvarwork)     
            CALL remap(index1,parain%npafdc,X,parain%pvarafdc)            
            CALL remap(index1,parain%npwage,X,parain%pvarwage)            
            IF (parain%cpregbig==1) CALL remap(index1,parain%nppreg,X,parain%pvarpreg)  
            IF (parain%cworkft==1) CALL remap(index1,parain%npwork,X,parain%pvarworkft)            
        END IF                
        IF (parain%ctype==1 .and. parain%ctypevar==1) THEN      ! only applicable if at least 2 types            
            call remap(index1,parain%nptype-1,X,parain%factort1(2:parain%typenum))  ! type-specific slope for first covariate
        elseif (parain%ctype==1 .and. parain%ctypevar==2) THEN      ! only applicable if at least 2 types            
            call remap(index1,parain%nptype-1,X,parain%factort1(2:parain%typenum))  ! type-specific slope for first covariate
            call remap(index1,parain%nptype-1,X,parain%factort2(2:parain%typenum))  ! type-specific slope for second covariate
        END IF
        
        IF (parain%cpreg2==1) then
            CALL remapsc(index1,X,parain%phn2)                 
        END IF
        
        IF (parain%cquad==1) THEN
            CALL remapsc(index1,X,temp)
            IF (parain%crra==0) THEN       
                parain%quadratic=temp * 0.0001d0
            ELSE
                parain%quadratic= EXP(temp-2.81d0)
            END IF            
        END IF                        
        
        IF (parain%cterm==1) THEN
            CALL remapsc(index1,X,parain%term_n1)
            CALL remapsc(index1,X,parain%term_n2)
            CALL remapsc(index1,X,parain%term_e1)
            CALL remapsc(index1,X,parain%term_e2)
        END IF
        
        IF (parain%cinc==1) THEN
            CALL remapsc(index1,X,parain%pyh)                             
        END IF        
        IF (parain%cinc2==1) THEN
            CALL remapsc(index1,X,parain%pyh2)                             
        END IF

        IF (parain%cinc3==1) THEN
            CALL remapsc(index1,X,parain%pyc) 
            CALL remapsc(index1,X,parain%pyp) 
        END IF
        IF (parain%cftpreg==1) THEN         ! allow for ft_work*preg preference
            CALL remapsc(index1,X,parain%errmean(8))
        END IF
        IF (parain%cptexp==1) THEN
            CALL remapsc(index1,X,parain%ptexp_share)
            parain%ptexp_share = blogit(parain%ptexp_share)
        END IF
        IF (parain%cplworkft==1) THEN
            CALL remapsc(index1,X,parain%plworkft)
        END IF
        IF (parain%cphn3==1) THEN
            CALL remapsc(index1,X,parain%phn3)
        END IF        
        IF (parain%cerrvar2==1) THEN
            CALL remapsc(index1,X,parain%errvar2)
            parain%errvar2=exp(parain%errvar2)
        END IF               
        if (parain%cworkn0==1) CALL remapsc(index1,X,parain%workn0)
        if (parain%cworkn1==1) CALL remapsc(index1,X,parain%workn1)
        if (parain%cworkn2==1) CALL remapsc(index1,X,parain%workn2)
        
        if (parain%cterm2==1) CALL remapsc(index1,X,parain%term_h)
        
        parain%paranum=index1   ! store total number of parameters        
        CONTAINS
            SUBROUTINE remap(index,nparam,X,Y)
                IMPLICIT NONE
                INTEGER,INTENT(INOUT) ::  index
                INTEGER,INTENT(IN)    ::  nparam            
                REAL(KIND=8),DIMENSION(:),INTENT(IN)  ::  X                        
                REAL(KIND=8),DIMENSION(:),INTENT(INOUT)  ::  Y
                Y(1:nparam) = X(index+1:index+nparam)
                index=index+nparam
            END SUBROUTINE remap        
            SUBROUTINE remapsc(index,X,Y)
                IMPLICIT NONE
                INTEGER,INTENT(INOUT) ::  index
                REAL(KIND=8),DIMENSION(:),INTENT(IN)  ::  X                        
                REAL(KIND=8),INTENT(INOUT)  ::  Y
                Y = X(index+1)
                index=index+1
            END SUBROUTINE remapsc            
    END SUBROUTINE remapparam
        
    SUBROUTINE LIKEFCN(N,X,F)
        use datastore, only: dataobj, paraobj, gdatain, parain, dpdatain, &  ! gdatain: data read in master program. parain: partially empty, for remapping parameters
             TID, OMP_GET_THREAD_NUM, CHUNKSIZE, NTHREAD, &         ! parallel computing parameters
             farr
        use likeparam                           ! load specific monitoring global variables for likelihood estimation
        use mkl95_blas          ! function NRM2
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  N           ! number of parameters
        REAL(KIND=8),INTENT(IN)    ::  X(N)    ! parameters
        REAL(KIND=8),INTENT(OUT)   ::  F       ! likelihood
        INTEGER    ::  iloop,jloop,mloop, dpiloop, dptloop, dptsize, typeloop, incloop
        INTEGER     ::  loopcount,loopindex(3,50),maxparloop
        farr=0.d0
        CALL remapparam(X,parain)        
        
        CALL OMP_SET_NUM_THREADS(nthread)                                              ! external function, set total number of threads
        IF (parain%cdynprog==1) THEN 
        
            IF (parain%creform==0 .OR. parain%creform==2) THEN
            !!$omp parallel default(none) shared(parain, dpdatain, gdatain)
            !TID = OMP_GET_THREAD_NUM()        
            DO dpiloop=1,parain%typecnt              ! number of observed types in the dp problem
                dpdatain(dpiloop,1)%evdatain=gdatain(dpiloop)%datain(1)
                dpdatain(dpiloop,1)%evdatain%varwork=0.d0   ! normalize work pref covariates
                dpdatain(dpiloop,1)%evdatain%varafdc=0.d0   ! normalize child care covariates
                dpdatain(dpiloop,1)%evdatain%varwage=0.d0   ! normalize wage eqn covariates
                dpdatain(dpiloop,1)%evdatain%varpreg=0.d0   ! normalize wage eqn covariates
                IF (parain%creform==0) THEN
                    CALL compevbase(parain,dpdatain(dpiloop,1),reform=1)          ! assume post-reform
                ELSE
                    CALL compevbase(parain,dpdatain(dpiloop,1),reform=0)          ! assume pre-reform
                END IF
            !$omp parallel default(none) shared(parain, dpdatain, gdatain, dpiloop,incloop,typeloop)      
                TID = OMP_GET_THREAD_NUM()        
            !$omp do schedule(static, chunksize)                                    
                do typeloop=1,parain%nptype
                    do incloop=1,parain%npinc
                    CALL dynprog (parain, dpdatain(dpiloop,1), typeloop, incloop)
                    end do
                end do
            !$omp end do                 
            !$omp end parallel                          
                dpdatain(dpiloop,2)=dpdatain(dpiloop,1)                
            END DO
            !!$omp end parallel          
            END IF            
            
            IF (parain%creform==1) THEN                 
                dpdatain(1,:)%evdatain=gdatain(1)%datain(1)                
                DO JLOOP=1,2           
                    dpdatain(1,jloop)%evdatain%varwork=0.d0   ! normalize work pref covariates
                    dpdatain(1,jloop)%evdatain%varafdc=0.d0   ! normalize child care covariates
                    dpdatain(1,jloop)%evdatain%varwage=0.d0   ! normalize wage eqn covariates
                    dpdatain(1,jloop)%evdatain%varpreg=0.d0   ! normalize wage eqn covariates                                                    
                    CALL compevbase(parain,dpdatain(1,jloop),reform=jloop-1)          ! PRE- AND POST-REFORM            
                end do
                
                loopcount=1
                do jloop=1,2
                    do typeloop=1,parain%nptype
                        do incloop=1,parain%npinc
                            loopindex(:,loopcount) = (/jloop,typeloop,incloop/)     ! record index of nested loops
                            loopcount=loopcount+1
                        end do
                    end do
                end do
                maxparloop = 2*parain%nptype*parain%npinc                                
                
            !$omp parallel default(none) shared(parain, dpdatain, gdatain,maxparloop,loopindex) private(jloop,typeloop,incloop,loopcount)
            TID = OMP_GET_THREAD_NUM()            
            !$omp do schedule(static)
            do loopcount=1,maxparloop
                jloop = loopindex(1,loopcount)
                typeloop = loopindex(2,loopcount)
                incloop = loopindex(3,loopcount)
                CALL dynprog (parain, dpdatain(1,jloop), typeloop, incloop)                     
            END DO
            !$omp end do
            !$omp end parallel    
            END IF                         
            
        END IF        
        
        CALL OMP_SET_NUM_THREADS(32)                                              ! external function, set total number of threads
        !$omp parallel default(none) shared(farr,gdatain,parain,dpdatain,iloop)
        !$omp do schedule(static)                                             ! start parallel do        
        do iloop=1,parain%groupsize                                                    ! loop variable is private by default
                CALL compliketype(farr(iloop), gdatain(iloop), parain, dpdatain(gdatain(iloop)%dptype,:))
        end do     
        !$omp end do
        !$omp end parallel           
        F=-SUM(farr(1:parain%groupsize))               
        
        IF ( currentmin==0.D0 .OR. ( (F<currentmin) .AND. nrm2(X-currentminarg(1:parain%paranum))>1.D-3 ) ) then
            currentmin=F   
            currentminarg(1:parain%paranum)=X    
            NITER=NITER+1
            write (1,*) 'iterate # and likelihood=', NITER, F
            PRINT *, 'iterate # and likelihood=', NITER, F                        
            write(1,'(100(5(f15.7, 1x), /))') (x(iloop), iloop=1,N)   
            if (parain%dirsim==1) write (1,'(f13.7)') farr(1:parain%groupsize)            
        END IF
    END SUBROUTINE LIKEFCN    
                            
    ! Compute likelihood each period. 
    SUBROUTINE COMPLIKEtype(out, gdataobji, parain, dpdatain)
        use anordf_int          ! computes normal cdf
        use bensim
        use datastore, only: mdim, dataobj, paraobj, gdataobj, lcobj, dpdataobj
        use platform, only: interpvec
        IMPLICIT NONE                
        TYPE(gdataobj),INTENT(INOUT)  ::  gdataobji  ! use intent(in) instead of intent(inout) modified 20121231 ! add posterior probs 20140602
        TYPE(paraobj),INTENT(IN)  ::  parain                     
        TYPE(dpdataobj),INTENT(IN) :: dpdatain(:)
        REAL(KIND=8), INTENT(OUT)  ::  out
        TYPE(lcobj) :: lc
        INTEGER    ::  iloop, kloop, jloop
        REAL(kind=8)  ::  jobprob, factorval, choicesum, wlnden, tempin(parain%ghq%npoint), temp(parain%typenum), &
            temparr(parain%nptype, parain%maxgsize) !temparr(parain%ghq%npoint, parain%maxgsize)
        REAL(KIND=8)    ::  factorvaltreat
        REAL(KIND=8),DIMENSION(parain%mdim)    ::  income, valuearr, evvalue, kernel
        REAL(KIND=8)    ::  wlnden2,wlnden3,pbias,disc,pbias2
        REAL(KIND=8),DIMENSION(parain%mdim)    ::  temp1,temp2,temp3
        REAL(KIND=8)        ::  lincomt(parain%nptype), typeprob(parain%nptype)
        INTEGER    ::  tlstart, reform        
        REAL(KIND=8)    ::  income_tmp(parain%mdim)
        REAL(KIND=8)    ::  evvalue_tmp(parain%mdim,parain%npinc)
        INTEGER     ::  incloop
        INTEGER     ::  currentreform
        tempin=0.d0
        temparr=0.d0
        temp=0.d0
        income=0.d0
        valuearr=0.d0       
        evvalue=0.d0 
        kernel=0.d0                
        disc = parain%discount
        IF (parain%creform==0) THEN
            tlstart = -1
        ELSEIF (parain%creform==2) THEN
            tlstart = 999
        ELSE
            tlstart = gdataobji%tlstart
        END IF        
        
        do kloop=1,gdataobji%gsize    
            IF (kloop == tlstart ) THEN                
                reform=1
                currentreform = 0
            ELSEIF (kloop .GT. tlstart) THEN
                reform=1
                currentreform = 1
            ELSE
                reform=0
                currentreform = 0
            END IF               
            IF (parain%ccurrentreform==0) currentreform=reform     ! no intermediate status
            
            CALL complincom(gdataobji%datain(kloop), parain, lc,0)                
            do iloop=1, parain%nptype
                IF (parain%ctype==1) CALL complincomf(lc, parain, iloop, ckid=gdataobji%datain(kloop)%ckid)                              
                do incloop=1,parain%npinc
                    CALL compev(gdataobji, parain, dpdatain(1+reform), kloop, iloop, incloop, evvalue_tmp(:,incloop) )        ! fill up the matrix for interpolation
                end do
            IF (parain%npinc==1) then
                evvalue = evvalue_tmp(:,1)
            else
                IF (parain%ccohabit==1) THEN
                    IF (gdataobji%datain(kloop)%varinc==0.d0) THEN          ! not cohabit yet
                        evvalue = interpvec(parain%npinc,parain%mdim,(/100.d0,200.d0,300.d0/),evvalue_tmp,0.d0)                   ! perform interpolation
                    ELSE                                                    ! cohabiting
                        evvalue = interpvec(parain%npinc,parain%mdim,(/100.d0,200.d0,300.d0/),evvalue_tmp,gdataobji%varinc)                   ! perform interpolation            
                    END IF
                ELSE
                    evvalue = interpvec(parain%npinc,parain%mdim,(/100.d0,200.d0,300.d0/),evvalue_tmp,gdataobji%varinc)                   ! perform interpolation          
                end if
            end if                                
                
                IF (lc%wage .LT. 1.d-4) THEN    ! non-worker
                    do jloop=1,parain%ghq%npoint
                        ! quadrature value of wage (given factor quadrature)
                        factorval=EXP(parain%ghq%absci(jloop) * lc%wstdlincomf + lc%wlincomf )
                        CALL geninc(income,factorval,0.d0,gdataobji%datain(kloop),reform=currentreform)   !u or u'
                        income = income + parain%pinc * gdataobji%datain(kloop)%varinc
                        income_tmp=(income) * (1.d0 + parain%pyh * (/0,1,1,0,1,1,0,1,1,0,1,1/) + parain%pyh2* (/0,0,1,0,0,1,0,0,1,0,0,1/) + parain%pyc* (/0,0,0,1,1,1,0,0,0,1,1,1/) + parain%pyp* (/0,0,0,0,0,0,1,1,1,1,1,1/) ) 
                        IF (lc%choicecase==0) THEN     ! no child care
                            valuearr=(lc%lincomf + income_tmp )/(parain%errvar2) + disc * evvalue          
                        ELSE
                            valuearr=(lc%lincomf + income_tmp )/(parain%errvar) + disc * evvalue        
                        END IF                        
                        CALL logit(kernel,valuearr)             
                        tempin(jloop)=kernel(lc%choice)
                    end do                    
                    temparr(iloop, kloop)=SUM(tempin * parain%ghq%weight)
                ELSE
                    CALL geninc(income,lc%wage ,0.d0, gdataobji%datain(kloop),reform=currentreform)    ! u or u'
                    income = income + parain%pinc * gdataobji%datain(kloop)%varinc
                    income_tmp=(income) * (1.d0 + parain%pyh * (/0,1,1,0,1,1,0,1,1,0,1,1/) + parain%pyh2* (/0,0,1,0,0,1,0,0,1,0,0,1/) + parain%pyc* (/0,0,0,1,1,1,0,0,0,1,1,1/) + parain%pyp* (/0,0,0,0,0,0,1,1,1,1,1,1/)  )    
                    IF (lc%choicecase==0) THEN
                        valuearr=(lc%lincomf + income_tmp )/(parain%errvar2) + disc * evvalue
                    ELSE
                        valuearr=(lc%lincomf + income_tmp )/(parain%errvar) + disc * evvalue
                    END IF                                            
                    CALL logit(kernel,valuearr)                                 
                    CALL lndeneasy(wlnden, LOG(lc%wage), lc%wlincomf , lc%wstdlincomf)
                    temparr(iloop, kloop) = kernel(lc%choice)*EXP(wlnden)
                END IF                                       

            end do        

            IF (parain%simcog==1) THEN
                CALL geninc(income,lc%wage ,0.d0, gdataobji%datain(kloop),reform=currentreform)
                IF ( IN(lc%choice, (/3,6,9,12/) ) ) THEN
                    gdataobji%datain(kloop)%siminc = income(lc%choice-1)
                ELSE
                    gdataobji%datain(kloop)%siminc = income(lc%choice)
                END IF
                IF (gdataobji%datain(kloop)%lpreg==1 .and. gdataobji%datain(kloop)%lwork==1 .and. parain%cleave==1) THEN
                    IF (parain%cleave_top==1) THEN
                        gdataobji%datain(kloop)%siminc = gdataobji%datain(kloop)%siminc + min(gdataobji%datain(kloop)%lwage * 0.8d0 * parain%leaveratio,240.d0)
                    ELSEIF (parain%cleave_top==0) THEN
                        gdataobji%datain(kloop)%siminc = gdataobji%datain(kloop)%siminc + gdataobji%datain(kloop)%lwage * 0.8d0 * parain%leaveratio
                    ELSE
                        gdataobji%datain(kloop)%siminc = gdataobji%datain(kloop)%siminc + min(gdataobji%datain(kloop)%lwage,240.d0) * 0.8d0 * parain%leaveratio - &
                            tftax(min(gdataobji%datain(kloop)%lwage,240.d0) * 0.8d0 * parain%leaveratio, gdataobji%datain(kloop)%ftax, 0 , 0)
                    END IF                    
                END IF
            END IF
            
        end do
        do iloop=1, parain%nptype
            temp(iloop)=PRODUCT(temparr(iloop,1:gdataobji%gsize ))                    
        end do            
                
        if (parain%ctype==1) then
            lincomt = parain%factort0 + parain%factort1*gdataobji%vart1 + parain%factort2*gdataobji%vart2
            call logit( typeprob, lincomt)            
            choicesum=SUM(temp(1:parain%nptype) * typeprob(1:parain%nptype) )                
        else
            choicesum=SUM(temp(1:parain%nptype))
        end if
        out=MAX(LOG(choicesum), -150.d0)  ! If too small (e.g. -100) then fjac may have huge values                
        do iloop=1, parain%nptype  ! add posterior 20140602
            gdataobji%posterior(iloop) = temp(iloop) * typeprob(iloop) / choicesum
        end do
     END SUBROUTINE COMPLIKEtype       
     
    SUBROUTINE compevbase(parain,dpdatain,reform) ! evbase and discount are the only outputs. datain is the only input.     
        use mkl95_blas    ! for gemv, DOT               
        USE datastore, only: dataobj,paraobj, dpdataobj, lcobj
        IMPLICIT NONE
        TYPE(paraobj),INTENT(IN)  ::  parain          ! use nptype,dpsdep,ghq
        TYPE(dpdataobj),INTENT(INOUT)  ::  dpdatain
        TYPE(lcobj)    ::  lc           ! lc is defined locally
        INTEGER                    ::  typeloop,hitloop,workloop,afdcloop,wageloop,cafdcloop,cwageloop, incloop
        REAL(KIND=8)                ::  factorval,income(parain%mdim), valuearr(parain%mdim)
        REAL(KIND=8)                ::  factorvaltreat
        INTEGER    ::  age1loop,age2loop,maxage1,maxage2
        INTEGER,INTENT(IN)        ::  reform
        REAL(KIND=8)        ::  income_tmp(parain%mdim)
        do cafdcloop=0,3                      ! limit state space from 1 to  (number of children)        
        IF (cafdcloop==0) THEN
            maxage2=0
            maxage1=0
        elseif (cafdcloop==1) then
            maxage2=0
            maxage1=4
        else
            maxage2=4
            maxage1=4
        end if
        do age2loop=0,maxage2
        do age1loop=age2loop,maxage1           
        
        do cwageloop=1,parain%maxcwageloop    ! limit state space from 1 to  (work experience)       
        do typeloop=1,parain%nptype
        do incloop=1,parain%npinc                          ! permanent income loop            
        if ( (age1loop==0 .and. cafdcloop==1) .or. (age2loop==0 .and. cafdcloop==2) ) then
            hitloop=2
        else
            hitloop=1
        end if
        CALL complincom(dpdatain%evdatain, parain, lc,1, ckid=cafdcloop,age1=age1loop,age2=age2loop, lpreg=hitloop-1)        ! initialize lc          
        do workloop=1,parain%dpsdep+1+1         ! lagged work (1,2,3) AND ALLOW FOR FT WORK AS STATE SPACE
            afdcloop=1
            CALL complincomfdyn(parain,lc,typeloop,hitloop,workloop,afdcloop,cafdcloop,cwageloop,age1loop,age2loop)  ! update lc according to state
            do wageloop=1,parain%ghq%npointdyn                    
                factorval=EXP(parain%ghq%abscidyn(wageloop) * lc%wstdlincomf + lc%wlincomf)
                factorvaltreat=factorval
                CALL geninc(income,factorval,factorvaltreat,dpdatain%evdatain, ckid=cafdcloop,age1=age1loop,age2=age2loop,sim=1,&
                    lpreg=hitloop-1,lwork=workloop-1, reform=reform) 
                income = income + parain%pinc * (incloop*100.d0)                ! each grid of perm income is 100000, 200000, 300000                
                income_tmp=(income) * (1.d0 + parain%pyh * (/0,1,1,0,1,1,0,1,1,0,1,1/) + parain%pyh2* (/0,0,1,0,0,1,0,0,1,0,0,1/) + parain%pyc* (/0,0,0,1,1,1,0,0,0,1,1,1/) + parain%pyp* (/0,0,0,0,0,0,1,1,1,1,1,1/) ) 
                
                IF (lc%choicecase==0) THEN     ! no child care
                    valuearr=(lc%lincomf + income_tmp)/(parain%errvar2)    
                ELSE
                    valuearr=(lc%lincomf + income_tmp)/(parain%errvar)    
                END IF
                
                dpdatain%evbase(cafdcloop,cwageloop,age1loop,age2loop)%evbase(typeloop,incloop,workloop,1,wageloop,:) = valuearr                
            end do                                            
        end do
        end do        
        end do 
        end do
        end do
        
        end do
        end do
    END SUBROUTINE compevbase       
     
    SUBROUTINE findevtp1 (dpdatain,parain,evtype,timeloop,typeloop,sloop,waitloop,cafdcloop,cwageloop,age1loop,age2loop,evtp1)
        use datastore, only: dpdataobj, paraobj    
        use platform, only: interpvec
        IMPLICIT NONE
        TYPE(dpdataobj),INTENT(IN)  ::  dpdatain      ! only evntl, ev, and evhit (some are never used)
        TYPE(paraobj),INTENT(IN)  ::  parain                
        INTEGER,INTENT(IN)    ::  evtype,timeloop,typeloop,cafdcloop, sloop,waitloop,cwageloop
        REAL(KIND=8),INTENT(OUT)   ::  evtp1(parain%mdim)
        INTEGER                    ::  jloop
        INTEGER,DIMENSION(parain%mdim)  ::  sarr,workarr,afdcarr,cafdcarr,cwagearr
        INTEGER    ::  nkid,workexp
        INTEGER,INTENT(IN)    ::  age1loop,age2loop
        INTEGER    ::  age1tp1, age2tp1, age1arr(parain%mdim), age2arr(parain%mdim)
        INTEGER ::  cwagearr0(parain%mdim)
        REAL(KIND=8)    ::  evtp1_tmp(parain%mdim,2)
               
        IF (parain%dpsdep==1) THEN
            workarr= (/1,2,3,1,2,3,1,2,3,1,2,3/)    ! lagged work       (0,1,2) ALLOW FOR FT WORK IN STATE SPACE            
            afdcarr= (/1,1,1,1,1,1,1,1,1,1,1,1/)    ! lagged child care
            sarr= (/1,1,1,1,1,1,1,1,1,1,1,1/)    ! lagged pregnancy
            nkid=MIN(cafdcloop+1,3)
            cafdcarr= (/cafdcloop,cafdcloop,cafdcloop,cafdcloop,cafdcloop,cafdcloop,nkid,nkid,nkid,nkid,nkid,nkid /)  ! number of children
            workexp=MIN(cwageloop+1,parain%maxcwageloop)
            IF (cwageloop==0) THEN
                cwagearr= (/1,1,1,1,1,1,1,1,1,1,1,1/) 
                cwagearr0= (/1,1,1,1,1,1,1,1,1,1,1,1/)
            ELSE
                cwagearr= (/cwageloop,workexp,workexp,cwageloop,workexp,workexp,cwageloop,workexp,workexp,cwageloop,workexp,workexp /)
                cwagearr0= (/cwageloop,cwageloop,workexp,cwageloop,cwageloop,workexp,cwageloop,cwageloop,workexp,cwageloop,cwageloop,workexp /) 
            END IF
            
        ELSE
        END IF            
        age1tp1 = min(age1loop+1,4)
        age2tp1 = min(age2loop+1,4)
        if (cafdcloop==0) then     ! no children now
            age1arr = (/ 0,0,0,0,0,0, 0,0,0,0,0,0 /)
            age2arr = (/ 0,0,0,0,0,0, 0,0,0,0,0,0 /)
        elseif (cafdcloop==1) then
            age1arr = (/ age1tp1,age1tp1,age1tp1,age1tp1,age1tp1,age1tp1, age1tp1,age1tp1,age1tp1,age1tp1,age1tp1,age1tp1 /)
            age2arr = (/ 0,0,0,0,0,0, 0,0,0,0,0,0 /)
        elseif (cafdcloop==2 .or. cafdcloop==3) then
            age1arr = (/ age1tp1,age1tp1,age1tp1,age1tp1,age1tp1,age1tp1, age1tp1,age1tp1,age1tp1,age1tp1,age1tp1,age1tp1 /)            
            age2arr = (/ age2tp1,age2tp1,age2tp1,age2tp1,age2tp1,age2tp1, age2tp1,age2tp1,age2tp1,age2tp1,age2tp1,age2tp1 /)
        else
            print *,'error tp1'
            stop
        end if
        evtp1_tmp(:,1) = (/ (dpdatain%ev(age1arr(jloop), age2arr(jloop))%ev(timeloop+1, typeloop, sloop, workarr(jloop), &
            afdcarr(jloop), cafdcarr(jloop),cwagearr0(jloop) ), jloop=1,parain%mdim) /)             ! pt experience has no value
        evtp1_tmp(:,2) = (/ (dpdatain%ev(age1arr(jloop), age2arr(jloop))%ev(timeloop+1, typeloop, sloop, workarr(jloop), &
            afdcarr(jloop), cafdcarr(jloop),cwagearr(jloop) ), jloop=1,parain%mdim) /)              ! pt same as ft experience
        evtp1 = interpvec(2,parain%mdim,(/0.d0,1.d0/),evtp1_tmp,parain%ptexp_share)                   ! perform interpolation
    END SUBROUTINE findevtp1
             
     SUBROUTINE dynprog(parain, dpdatain, typeloop,incloop)
        !use mkl_vml
        use mkl95_blas     
        use datastore, only: paraobj, dpdataobj
        IMPLICIT NONE                
        TYPE(paraobj),INTENT(IN)  ::  parain 
        TYPE(dpdataobj),INTENT(INOUT)   ::  dpdatain   
        INTEGER,INTENT(IN)    ::  typeloop
        INTEGER,INTENT(IN)  ::  incloop
        INTEGER    ::  timeloop, sloop, waitloop, workloop, afdcloop,cafdcloop,cwageloop
        REAL(KIND=8)    ::  evtemp(parain%mdim)
        INTEGER    :: hittemp
        REAL(KIND=8)    ::  temp=0.d0        
        INTEGER    ::  maxage1,maxage2,age1loop,age2loop        
        ! initialize terminal values 201608
        do workloop=1,parain%dpsdep+1 +1
            do cafdcloop=0,3
                do cwageloop=1,parain%maxcwageloop
                    do age2loop=0,4
                        do age1loop=0,4
                            dpdatain%ev(age1loop,age2loop)%ev(dpdatain%tlhor,:,:,workloop,1,cafdcloop,cwageloop) = parain%term_n1*cafdcloop + &
                                parain%term_n2*(cafdcloop**2) + parain%term_e1*cwageloop + parain%term_e2*(cwageloop**2) + parain%term_h*indic(workloop .GT. 1)
                        end do
                    end do                    
                end do
            end do
        end do
        
        
        do timeloop=dpdatain%tlhor-1, 1, -1         
            do cafdcloop=0,3      ! number of kids            
            IF (cafdcloop==0) THEN
                maxage2=0
                maxage1=0
            elseif (cafdcloop==1) then
                maxage2=0
                maxage1=4
            else
                maxage2=4
                maxage1=4
            end if
            do age2loop=0,maxage2
            do age1loop=age2loop,maxage1                                                                   
            sloop=1
            do cwageloop=1,parain%maxcwageloop      ! work experience
                CALL findevtp1(dpdatain,parain,1,timeloop,typeloop,incloop,-1,cafdcloop,cwageloop,age1loop,age2loop,evtemp)                    
                do workloop=1,parain%dpsdep+1+1       ! lagged work  (0,1,2) ALLOW FOR FT WORK IN STATE SPACE
                    afdcloop=1
                    temp=0.d0
                    dpdatain%ev(age1loop,age2loop)%ev(timeloop,typeloop,incloop,workloop,afdcloop,cafdcloop,cwageloop)=&
                        evrecur(dpdatain,parain,evtemp,typeloop,incloop,workloop,afdcloop,cafdcloop,cwageloop,age1loop,age2loop) - temp
                end do
            end do      
            end do   
            end do
            end do
        end do           
        CONTAINS
            ! compute emax (i.e. ev) given state, u_k and ev_k_t+1
            REAL(KIND=8) PURE FUNCTION evrecur(dpdatain, parain, evtp1, typeloop, hitloop, workloop, afdcloop, cafdcloop,cwageloop,age1loop,age2loop)            
                use mkl95_blas    ! for DOT
                use datastore, only: dpdataobj, paraobj
                IMPLICIT NONE
                TYPE(dpdataobj),INTENT(IN)  ::  dpdatain   ! evbase as the only input
                TYPE(paraobj),INTENT(IN)  ::  parain       ! discount, ghq
                REAL(KIND=8),INTENT(IN)    ::  evtp1(parain%mdim)  ! next period ev                   
                INTEGER,INTENT(IN)    ::  typeloop,hitloop,workloop,afdcloop,cafdcloop,cwageloop
                REAL(KIND=8)    ::  valuearr(parain%mdim), tempin(parain%ghq%npointdyn), tempintmp, kernel(parain%mdim), pbias,disc,pbias2, TEMPINTMP1
                INTEGER    ::  wageloop
                INTEGER,INTENT(IN)    ::  age1loop,age2loop
                do wageloop=1,parain%ghq%npointdyn                    
                    valuearr=dpdatain%evbase(cafdcloop,cwageloop,age1loop,age2loop)%evbase(typeloop, hitloop, workloop, afdcloop, wageloop, :) &
                        + parain%discount * evtp1
                    tempintmp     =parain%ghq%euler + MAXVAL(valuearr) + LOG( SUM( EXP(valuearr - MAXVAL(valuearr) ) ) ) ! avoid overflow
                    tempin(wageloop) = tempintmp
                end do                    
                evrecur = SUM(tempin * parain%ghq%weightdyn)
            END FUNCTION evrecur             
    END SUBROUTINE dynprog
           
    ! Finds ev in period t+1 (for compltype or simutype only): given time and type, load state and determine next period's ev
    SUBROUTINE compev(gdataobji,parain,dpdatain,timeloop,typeloop,incloop,evvalue, evfirst)
        USE datastore, only: gdataobj, paraobj, dpdataobj
        use platform, only: interpvec
        IMPLICIT NONE
        TYPE(gdataobj), INTENT(IN) :: gdataobji
        TYPE(paraobj),INTENT(IN)  ::  parain
        TYPE(dpdataobj),INTENT(IN) :: dpdatain
        INTEGER,INTENT(IN)    ::  timeloop,typeloop,incloop
        REAL(KIND=8),INTENT(OUT) ::  evvalue(parain%mdim)        
        INTEGER    ::  timelooptmp, count1, count2 , cafdctemp, cwagetemp, cwageflag, cwagetemp1, cwagetemp2
        REAL(KIND=8),DIMENSION(parain%mdim)    ::  evvalue1, evvalue2
        INTEGER,INTENT(IN),OPTIONAL  ::  evfirst
        INTEGER    ::  age1loop, age2loop
        REAL(KIND=8)    ::  workexp, evvalue_tmp(parain%mdim,2)
        timelooptmp = timeloop    
        age1loop = INT(gdataobji%datain(timeloop)%age1)
        age2loop = INT(gdataobji%datain(timeloop)%age2)
        workexp = gdataobji%datain(timeloop)%workexp_ft + (gdataobji%datain(timeloop)%workexp_pt)*parain%ptexp_share      ! compute units of work experience given data            
        CALL findevtp1(dpdatain,parain,1,timelooptmp,typeloop,incloop,-1,INT(gdataobji%datain(timeloop)%ckid),MIN(INT(workexp)  , parain%maxcwageloop), &
            age1loop, age2loop, evvalue_tmp(:,1)) ! ev or evhit                
        CALL findevtp1(dpdatain,parain,1,timelooptmp,typeloop,incloop,-1,INT(gdataobji%datain(timeloop)%ckid),MIN(INT(workexp)+1, parain%maxcwageloop), &
            age1loop, age2loop, evvalue_tmp(:,2)) ! ev or evhit                        
        evvalue = interpvec(2,parain%mdim,(/0.d0,1.d0/),evvalue_tmp,workexp-INT(workexp))                   ! perform interpolation                
        evvalue = evvalue - evvalue(1)      ! normalize expected value function
    END SUBROUTINE compev             

    ! Simulate behaviors of individuals and write output
     SUBROUTINE simulatetype(gdataobji, parain, dpdatain, ind, simsize, writenum)
        use mkl95_blas          ! function DOT
        !use mkl95_lapack
        use datastore, only: paraobj, gdataobj, lcobj, dpdataobj
        USE platform,   only: interpvec
        IMPLICIT NONE                
        TYPE(gdataobj),INTENT(INOUT)  ::  gdataobji
        TYPE(paraobj),INTENT(IN)  ::  parain
        TYPE(dpdataobj),INTENT(IN) :: dpdatain(:)
        INTEGER,INTENT(IN)    ::  ind, simsize, writenum  ! ind: program availability indicator (default=4)
        TYPE(lcobj) :: lc        
        INTEGER    ::  factorvalout, iloop, jloop, kloop, mloop, choice
        REAL(KIND=8)    ::  jobprob, wage, randwage, utility !, ben(8,4)        
        REAL(KIND=8), DIMENSION(parain%mdim)   ::  income, kernel, valuearr, evvalue   
        REAL(KIND=8)    ::  partiarr(4), hourwage, workscale
        REAL(KIND=8)    ::  wlin, hypwage, hypwage0, hypwage1, hypwage2, hypwage3
        INTEGER        ::  jidx, year 
        REAL(KIND=8)    ::  randwerr, obswage, obschoice, pbias,disc, pbias2
        REAL(KIND=8)    ::  wagetreat
        REAL(KIND=8), DIMENSION(parain%mdim)    ::  ben
        REAL(KIND=8), DIMENSION(parain%mdim,5) ::  benmat
        INTEGER        ::  timesize
        REAL(KIND=8)    ::  evt1
        real(kind=8)    :: benscalar=0.d0
        INTEGER    ::  tlstart,reform       
        REAL(KIND=8)    ::  var1,var2 
        REAL(KIND=8)    ::  income_tmp(parain%mdim), typeprob(parain%nptype), lincomt(parain%nptype)
        INTEGER     ::  incloop
        REAL(KIND=8),DIMENSION(parain%mdim,parain%npinc)    ::  EVVALUE_TMP
        INTEGER     ::  currentreform
        INTEGER     ::  intout
        evvalue=0.d0        ! initialize
        IF (parain%longsim==1) THEN
            timesize=parain%maxgsize
        ELSE
            timesize=gdataobji%gsize
        END IF
        IF (parain%creform==0) THEN
            tlstart = -1
        ELSEIF (parain%creform==2) THEN
            tlstart = 999
        ELSE
            tlstart = gdataobji%tlstart
        END IF        
        
        do kloop=gdataobji%gsize+1, parain%maxgsize     ! initialize post-sample covariates
            gdataobji%datain(kloop)%varafdc=gdataobji%datain(gdataobji%gsize)%varafdc
            gdataobji%datain(kloop)%varwork=gdataobji%datain(gdataobji%gsize)%varwork
            gdataobji%datain(kloop)%varwage=gdataobji%datain(gdataobji%gsize)%varwage
            gdataobji%datain(kloop)%varpreg=gdataobji%datain(gdataobji%gsize)%varpreg
            IF (gdataobji%datain(KLOOP-1)%varinc .GT. 0.d0) THEN            ! cohabited last period
                gdataobji%datain(kloop)%varinc = gdataobji%datain(KLOOP-1)%varinc
            ELSE            ! not cohabit last period
                intout = catrandnum(parain%sstream3, (/parain%mrate, 1.d0-parain%mrate/)     )   ! some chance of getting cohabited
                IF (intout==1) THEN
                    gdataobji%datain(kloop)%varinc = gdataobji%varinc
                ELSE
                    gdataobji%datain(kloop)%varinc=0.d0     ! assume everyone gets married
                END IF
            END IF            
        end do

        if (parain%ctype==1) then
            lincomt = parain%factort0 + parain%factort1*gdataobji%vart1 + parain%factort2*gdataobji%vart2
            call logit( typeprob, lincomt)            
        end if                              
        
        do mloop=1,simsize                                           ! simulate paths for individual            
        factorvalout=catrandnum(parain%sstream, typeprob)     ! randomly generate type : 1, 2, 3, etc.  
        do kloop=1, TIMESIZE                    ! simulation starts from WAVE 2 (kloop=1)
            IF (kloop == tlstart ) THEN                
                reform=1
                currentreform = 0
            ELSEIF (kloop .GT. tlstart) THEN
                reform=1
                currentreform = 1
            ELSE
                reform=0
                currentreform = 0
            END IF               
            IF (parain%ccurrentreform==0) currentreform=reform     ! no intermediate status
            CALL complincom(gdataobji%datain(kloop), parain, lc,0)
            IF (parain%ctype==1) CALL complincomf(lc, parain, factorvalout, ckid=gdataobji%datain(kloop)%ckid)
            do incloop=1,parain%npinc
                CALL compev(gdataobji, parain, dpdatain(1+reform), kloop, factorvalout, incloop, evvalue_tmp(:,incloop) )        ! fill up the matrix for interpolation
            end do
            IF (parain%npinc==1) then
                evvalue = evvalue_tmp(:,1)
            else
                IF (parain%ccohabit==1) THEN
                    IF (gdataobji%datain(kloop)%varinc==0.d0) THEN          ! not cohabit yet
                        evvalue = interpvec(parain%npinc,parain%mdim,(/100.d0,200.d0,300.d0/),evvalue_tmp,0.d0)                   ! perform interpolation
                    ELSE                                                    ! cohabiting
                        evvalue = interpvec(parain%npinc,parain%mdim,(/100.d0,200.d0,300.d0/),evvalue_tmp,gdataobji%varinc)                   ! perform interpolation            
                    END IF
                ELSE
                    evvalue = interpvec(parain%npinc,parain%mdim,(/100.d0,200.d0,300.d0/),evvalue_tmp,gdataobji%varinc)                   ! perform interpolation          
                end if                          
            end if
            
            randwage=randnum(parain%sstream, 'N')                           ! randomly generate wage offer
            wage=EXP( randwage * lc%wstdlincomf + lc%wlincomf)
            wage=MIN(wage,4000.d0)
            wagetreat=wage
            IF (parain%writesim==3) THEN
                CALL geninc(income,wage,wagetreat,gdataobji%datain(kloop), ben=ben, BENMAT=BENMAT,reform=currentreform)
            ELSE
                CALL geninc(income,wage,wagetreat,gdataobji%datain(kloop), ben=ben,reform=currentreform)               
            END IF            
            income = income + parain%pinc * gdataobji%datain(kloop)%varinc
            income_tmp=(income) * (1.d0 + parain%pyh * (/0,1,1,0,1,1,0,1,1,0,1,1/) + parain%pyh2* (/0,0,1,0,0,1,0,0,1,0,0,1/) + parain%pyc* (/0,0,0,1,1,1,0,0,0,1,1,1/) + parain%pyp* (/0,0,0,0,0,0,1,1,1,1,1,1/)    )
            valuearr=(lc%lincomf + income_tmp) /(parain%errvar) + parain%discount * evvalue                                                 
            CALL logit(kernel,valuearr)                                              
            
            choice = catrandnum(parain%sstream2, kernel)                    ! randomly generate choice given choice probabilities
            IF (choice==1 .or. choice==4 .or. choice==7 .or. choice==10) THEN
                obswage = 0.D0
            ELSE
                obswage = wage
            END IF            
            
            IF ( IN(choice,(/2,5,8,11/)) ) THEN
                benscalar = income(choice) - wage - parain%pinc * gdataobji%datain(kloop)%varinc
            ELSEIF ( IN(choice,(/3,6,9,12/)) ) THEN
                benscalar = income(choice-1) - wage - parain%pinc * gdataobji%datain(kloop)%varinc
            ELSE
                benscalar = income(choice)- parain%pinc * gdataobji%datain(kloop)%varinc
            END IF                        
                        
            IF (parain%simupdate==1 .or. (parain%simupdate==2 .and. (kloop .gt. gdataobji%tlstart) )) THEN            ! directive: update state variables
                ! UPDATE STATE VARIABLES (LAGGED DEPENDENT VARIABLES; CUMAFDC; CUMHITWAIT; HIT)
                IF (kloop<TIMESIZE) then
                    IF (parain%dpsdep==1) THEN                 ! If contain lagged variables then update lags.
                        gdataobji%datain(kloop+1)%lworkft    = indic( IN( choice,(/3,6,9,12/) ) )      ! lagged ft work status                        
                        gdataobji%datain(kloop+1)%lwork    = indic( IN( choice,(/2,3,5,6,8,9,11,12/) ) ) 
                        gdataobji%datain(kloop+1)%lcare    = indic( IN( choice,(/4,5,6,10,11,12/) ) )
                        gdataobji%datain(kloop+1)%lpreg    = indic( IN( choice,(/7,8,9,10,11,12/) ) )
                        IF (IN( choice,(/1,2,3,4,5,6/) ) ) THEN      ! not pregnant
                            gdataobji%datain(kloop+1)%ckid    = gdataobji%datain(kloop)%ckid
                        ELSE
                            gdataobji%datain(kloop+1)%ckid    = MIN(gdataobji%datain(kloop)%ckid+1.D0,3.d0)
                        END IF
                        IF (IN( choice,(/1,4,7,10/) ) ) THEN    ! no work
                            gdataobji%datain(kloop+1)%cwage    = gdataobji%datain(kloop)%cwage
                            gdataobji%datain(kloop+1)%workexp_pt    = gdataobji%datain(kloop)%workexp_pt
                            gdataobji%datain(kloop+1)%workexp_ft    = gdataobji%datain(kloop)%workexp_ft
                        ELSEIF (IN(choice, (/2,5,8,11/) ))  THEN   ! part time work
                            gdataobji%datain(kloop+1)%cwage         = MIN(gdataobji%datain(kloop)%cwage+parain%ptexp_share,DBLE(parain%maxcwageloop))
                            gdataobji%datain(kloop+1)%workexp_pt    = MIN(gdataobji%datain(kloop)%workexp_pt+1.D0,DBLE(parain%maxcwageloop))
                            gdataobji%datain(kloop+1)%workexp_ft    = gdataobji%datain(kloop)%workexp_ft
                        ELSEIF (IN(choice, (/3,6,9,12/) ))  THEN   ! part time work
                            gdataobji%datain(kloop+1)%cwage         = MIN(gdataobji%datain(kloop)%cwage+1.D0,DBLE(parain%maxcwageloop))
                            gdataobji%datain(kloop+1)%workexp_pt    = gdataobji%datain(kloop)%workexp_pt                                                        
                            gdataobji%datain(kloop+1)%workexp_ft    = MIN(gdataobji%datain(kloop)%workexp_ft+1.D0,DBLE(parain%maxcwageloop))
                        END IF                                        
                        IF (gdataobji%datain(kloop)%ckid==0) THEN
                            gdataobji%datain(kloop+1)%age1 = 0.D0
                            gdataobji%datain(kloop+1)%age2 = 0.D0
                        ELSEIF (gdataobji%datain(kloop)%ckid==1) THEN
                            gdataobji%datain(kloop+1)%age1 = MIN(gdataobji%datain(kloop)%age1+1.D0,4.D0)
                            gdataobji%datain(kloop+1)%age2 = 0.D0
                        ELSEIF (gdataobji%datain(kloop)%ckid > 1) THEN
                            gdataobji%datain(kloop+1)%age1 = MIN(gdataobji%datain(kloop)%age1+1.D0,4.D0)
                            gdataobji%datain(kloop+1)%age2 = MIN(gdataobji%datain(kloop)%age2+1.D0,4.D0)
                        END IF          
                        CALL gencase(INT(gdataobji%datain(kloop+1)%ckid), INT(gdataobji%datain(kloop+1)%age1), INT(gdataobji%datain(kloop+1)%age2), &
                            gdataobji%datain(kloop+1)%choicecase, var1, var2)
                    END IF                   
                END IF
            END IF
            ! Concurrent emax utility (no euler's constant; level not matter)
            utility =  ( MAXVAL(valuearr) + LOG( SUM( EXP( valuearr-MAXVAL(valuearr) ) ) ) - DOT(kernel,parain%discount*evvalue) )*parain%errvar ! avoid overflow
            
            ! Write output: id, sim#, time, type, choice, wage, benefit (4x), utility, cumafdc, hit, cumhitwait
            if (parain%writesim==1) then
                write(writenum, '(i8, 1x, 5(i2, 1x), 3(f8.2,1x), 7(f4.1,1x), (i2,1x), 5(f8.2,1x), (f8.2,1x) (f13.7,1x) )') gdataobji%id, &
                    mloop, kloop+1, factorvalout, parain%lcftype, choice, &           ! output kloop+1 (wave 2,3,...) instead of kloop
                    utility, wage, obswage, &
                    gdataobji%datain(kloop)%ckid, gdataobji%datain(kloop)%cwage, &
                    gdataobji%datain(kloop)%lwork, gdataobji%datain(kloop)%lcare, gdataobji%datain(kloop)%lpreg, &
                    gdataobji%datain(kloop)%age1, gdataobji%datain(kloop)%age2, &
                    gdataobji%tlstart, benscalar, ben(choice), &
                    gdataobji%datain(kloop)%varinc          ! output partner's income 20170109
                    !, parain%factorw(2), parain%factorw(3), gdataobji%posterior(2), gdataobji%posterior(3)
                    !gdataobji%tlstart, benscalar !, parain%factorw(2), parain%factorw(3), gdataobji%posterior(2), gdataobji%posterior(3)                        
            end if
        end do 
        end do
        
        if (parain%simcog==1) THEN      
            do iloop=1,parain%typenum     
                write (100, '(i8, 1x, 10(f8.4,1x) )') gdataobji%id, parain%factorw(iloop), gdataobji%posterior(iloop)
            end do
        end if
        
    END SUBROUTINE simulatetype  
    
    SUBROUTINE update(constant, increment)     
        IMPLICIT NONE
        REAL(KIND=8),INTENT(INOUT)   ::  constant
        REAL(KIND=8),INTENT(IN)    ::  increment
        constant=constant+increment
    END SUBROUTINE update  
    
    SUBROUTINE LIKEVFCN(M,N,X,F)
        !use subroutines, ONLY: likefcn
        USE DATASTORE, ONLY:   farr
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  M,N
        REAL(KIND=8),INTENT(IN)  ::  X(N)
        REAL(KIND=8),INTENT(OUT) ::  F(M)        
        REAL(KIND=8)  ::  fvalue
        CALL LIKEFCN(N,X,fvalue)
        F=-farr
    END SUBROUTINE LIKEVFCN    
    SUBROUTINE LIKEVFCN_std(M,N,X,F)
        !use subroutines, ONLY: likefcn
        USE DATASTORE, ONLY:   farr, farrindex1, farrindex2
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  M,N
        REAL(KIND=8),INTENT(IN)  ::  X(N)
        REAL(KIND=8),INTENT(OUT) ::  F(M)        
        REAL(KIND=8)  ::  fvalue
        CALL LIKEFCN(N,X,fvalue)
        F=-farr(farrindex1:farrindex2)
    END SUBROUTINE LIKEVFCN_std
    

    FUNCTION inverse(A,N)
        USE LINRG_INT
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  N   ! Gives the dimension of the input matrix
        REAL(KIND=8),DIMENSION(N,N)    ::  inverse            
        REAL(KIND=8),DIMENSION(N,N),INTENT(IN)  ::  A
        CALL LINRG(A,inverse)
    END FUNCTION inverse 
            
    SUBROUTINE std(paravec,parastd,covmat,datasize,paranum, fjac)
        USE FDJAC_INT
        USE MXTXF_INT
        USE datastore, only: path, farrindex1, farrindex2
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  datasize,paranum                
        REAL(KIND=8),INTENT(IN)  ::  paravec(paranum)
        REAL(KIND=8),INTENT(OUT) ::  parastd(paranum)     
        REAL(KIND=8),INTENT(OUT)   ::  covmat(paranum,paranum)
        !REAL(KIND=8)  ::  vfvalue(datasize), fjac(datasize,paranum), xscale(paranum)
        REAL(KIND=8)  ::  vfvalue(datasize), xscale(paranum)        
        REAL(KIND=8),dimension(:,:),intent(inout) ::  fjac        
        REAL(KIND=8)    ::  temp(paranum,paranum)
        LOGICAL    ::  singulararr(paranum) , singular=.FALSE.
        INTEGER    ::  iloop,jloop
        INTEGER,PARAMETER   ::  jacsize=1000
        real(kind=8),dimension(jacsize,paranum)  ::  FJACTMP           ! address stack overflow problem (temporary solution)
        integer ::  COUNT,LOOPNUM,res
        real(kind=8)    ::  DJACOBI
        !REAL(KIND=8),DIMENSION(1)           ::  vfvaluetmp
        singulararr=.false.
        CALL likevfcn(datasize,paranum,paravec,vfvalue)
        xscale = ABS(1.D0/paravec)   ! define scale for std err calculation            
        
        loopnum = INT(real(DATASIZE)/jacsize)
        print *,'loopnum is ', loopnum, 'datasize is', datasize
        count = 0
        do iloop=1,loopnum
            farrindex1=count+1
            farrindex2=count+jacsize
            CALL FDJAC(likevfcn_std, paravec, vfvalue(farrindex1:farrindex2), fjactmp, XSCALE=xscale, EPSFCN=0.00D0)
            fjac(farrindex1:farrindex2,1:paranum) = fjactmp
            count = count+jacsize
            PRINT *,COUNT
        end do
        if (datasize>count) then
            print *,'count is', count
            farrindex1=count+1
            farrindex2=datasize            
            CALL FDJAC(likevfcn_std, paravec, vfvalue(farrindex1:farrindex2), fjactmp(1:farrindex2-farrindex1+1,1:paranum), XSCALE=xscale, EPSFCN=0.00D0)
            !CALL FDJAC(likevfcn, paravec, vfvalue(count+1:datasize), fjactmp, XSCALE=xscale, EPSFCN=0.00D0)
            fjac(farrindex1:farrindex2,1:paranum) = fjactmp(1:farrindex2-farrindex1+1,1:paranum)
            print *,'fdjac computation done!'
        end if
        open(30, file=trim(path)//'fjac.txt',status='replace')
        DO iloop=1,datasize
            WRITE(30,'(100f15.7)') (fjac(iloop,jloop), jloop=1,paranum)
        END DO         
        close(30)
        DO jloop=1,paranum            
            IF (SUM(ABS( fjac(:,jloop))) < 0.0000001D0) THEN           ! determine singularity
                singulararr(jloop)=.TRUE.
                singular=.TRUE.
            END IF
        END DO
        IF (singular==.FALSE.) THEN            ! invert matrix if not singular
            !CALL MXTXF (fjac, temp)
            !CALL MXTXF (fjac(1:datasize,1:paranum), temp)
            temp = matmul(transpose(fjac),fjac)
            covmat = inverse(temp, paranum)
            !stdmat = inverse( matmul(transpose(fjac), fjac), paranum )
            DO iloop=1,paranum
                parastd(iloop) = sqrt(covmat(iloop, iloop))
            END DO
        ELSE
            CALL BEEPQQ (4000,300)    
            DO iloop=1,paranum
                IF (singulararr(iloop)==.TRUE.) THEN 
                    parastd(iloop) = 111.D0     ! singular
                ELSE
                    parastd(iloop) = 0.d0     ! not singular
                END IF
            END DO
        END IF
    END SUBROUTINE std                 
             
     REAL(KIND=8) FUNCTION randnum(stream, rtype)
        use mkl_vsl
        use mkl_vsl_type
        IMPLICIT NONE
        TYPE (VSL_STREAM_STATE), INTENT(IN) :: stream
        CHARACTER(1),INTENT(IN)   ::  rtype
        REAL(KIND=8)    ::  temp(1)
        INTEGER    ::  iostat
        IF (rtype == 'N') THEN
            iostat=vdrnggaussian( VSL_METHOD_DGAUSSIAN_BOXMULLER2, stream, 1, temp, 0.d0, 1.d0 )
        ELSEIF (rtype == 'U') THEN
            iostat=vdrnguniform( VSL_METHOD_DUNIFORM_STD_ACCURATE, stream, 1, temp, 0.d0, 1.d0 )
        ELSE 
            PRINT *, 'error randnum type "U or N"!'
        END IF
        if (iostat /= vsl_error_ok) print *, 'error produce stream'             
        randnum=temp(1)       
    END FUNCTION randnum
    
    INTEGER FUNCTION catrandnum(stream, probin)
        use mkl_vsl_type
        IMPLICIT NONE
        TYPE (VSL_STREAM_STATE),INTENT(IN) :: stream
        REAL(KIND=8), INTENT(IN)   ::  probin(:)
        INTEGER    ::  arrsize, iloop, done
        REAL(KIND=8)    ::  uniform, probarr(SIZE(probin))
        arrsize=size(probin)
        if (ABS(SUM(probin)-1.d0) >1.d-4 ) print *, 'error catrandnum'
        probarr(1)=probin(1)
        !if (arrsize>1) then
            do iloop=2,arrsize
                probarr(iloop)=probarr(iloop-1)+probin(iloop)
            end do        
        !end if
        uniform=randnum(stream, 'U')
        done=0
        do iloop=1,arrsize
            if (  ( uniform .LT. probarr(iloop) )    .AND. done==0) THEN
                catrandnum=iloop
                done=1
            end if
        end do        
        if (done==0) catrandnum=arrsize
    END FUNCTION catrandnum    
    
    LOGICAL FUNCTION IN(intin, intlist)
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  intin, intlist(:)
        INTEGER    ::  iloop       
        in=.false.
        do iloop=1,size(intlist)
            if (intin .EQ. intlist(iloop)) then
                in=.true.
                exit
            end if
        end do
    END FUNCTION IN
    PURE FUNCTION crrafn(income, param,N)
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  N
        REAL(KIND=8),INTENT(IN),DIMENSION(N)    ::  income
        REAL(KIND=8),INTENT(IN)    ::  param
        REAL(KIND=8),DIMENSION(N)   ::  crrafn
        REAL(KIND=8)    ::  temp
        temp = 1.d0-param
        crrafn=( (income+1.d0)**temp - 1.d0)/temp
        !crrafn=(EXP(temp*LOG(income+1.d0))-1.d0)/temp
    END FUNCTION crrafn
    PURE FUNCTION quadfn(income, param,N)
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  N
        REAL(KIND=8),INTENT(IN),DIMENSION(N)    ::  income
        REAL(KIND=8),INTENT(IN)    ::  param
        REAL(KIND=8),DIMENSION(N)   ::  quadfn
        REAL(KIND=8)    ::  temp
        quadfn = income + param*income**2.d0
    END FUNCTION quadfn        
    PURE REAL(KIND=8) FUNCTION blogit(x)
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  x
        IF (x>0.d0) THEN           ! AVOID OVERFLOW
            blogit = 1.d0 / ( 1.d0 + EXP(-x) )
        ELSE
            blogit = exp(x) / ( 1.d0 + EXP(x) )
        END IF
    END FUNCTION blogit    
    PURE FUNCTION indic(logic)
        IMPLICIT NONE
        LOGICAL,INTENT(IN)    ::  logic
        REAL(KIND=8)    ::  indic
        IF (logic .EQ. .TRUE.) THEN
            indic = 1.D0
        ELSE
            indic = 0.D0
        END IF
    END FUNCTION indic
    PURE SUBROUTINE logit(out,arr,logicarr)
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  arr(:)
        LOGICAL,optional, INTENT(IN)    ::  logicarr(:)
        REAL(KIND=8),INTENT(OUT)   ::  out(:)
        REAL(KIND=8)    ::  base, nomin(size(arr)), arrnorm(size(arr))
        arrnorm=arr-MAXVAL(arr)          ! normalize to avoid overflow
        nomin=EXP(arrnorm)              ! array of numerators
        base=SUM(nomin)                 ! sum of array of numerators
        out=nomin/base
    END SUBROUTINE logit      
    
        
END MODULE SUBROUTINES