*Codes for "Life-Cycle and Intergenerational Effects of Child Care Reforms" published in Quantitative Economics
* copyright by Marc K. Chan and Kai Liu, 2017
* generates all test score regression results

*note: child2(dummy=1 if the child is a 2nd child)
*note: nchild (family size =1,=2,=3)
cap log close
log using score_2016_short,replace

* use all sample (child2 and child1)
use simcog_rme_1_high_v1_c2,clear
append using simcog_rme_1_high_v1
replace meduc=1
append using simcog_rme_1_low_v1_c2
append using simcog_rme_1_low_v1
replace child2=0 if child2==.

*define school quintiles
egen qt05= cut(avg_score05), at(-10, 63.46, 66.92, 69.94, 73.55, 10000)
egen qt08= cut(avg_score08), at(-10, 78.76, 82.50, 85.62, 90.69, 10000)
egen qt05_grp = group(qt05)
egen qt08_grp = group(qt08)
gen qt = qt05_grp if qt05_grp !=.
replace qt= qt08_grp if qt08_grp !=.

*normalize preferences (not essential)
gen emu_work_1=(emu_work-426)/100 if meduc==0
replace emu_work_1=(emu_work-590)/100 if meduc==1
gen emu_preg_1=(emu_preg-466)/100 if meduc==0
replace emu_preg_1=(emu_preg-775)/100 if meduc==1

gen anyworkall_m = anyworkall*meduc
gen careall_new_m = careall_new*meduc
gen wdall_m = wdall*meduc
gen wdcall_m = wdcall*meduc
gen best2all_m = best2all*meduc
gen siminc_m = siminc_both_all_new*meduc

global reglist1 meduc fedu_hs male lnbweight1 mage1_yng mgap4 grandma child2 score1_d
global reglist2a emu_wage emu_work_1
global reglist2b emu_wage emu_preg_1
global reglist3a wdall wdcall siminc_both_all_new
global reglist3b wdall wdcall siminc_both_all_new wdall_m wdcall_m siminc_m

*save combined, replace

*note: best2all is (employed + formal care)

*****************************************
*Table 1: reading score, low education

eststo clear
*employment and child care, no control function
eststo: reg lnscore1_new anyworkall careall_new siminc_both_all_new $reglist1 if score_type==1 & meduc==0
*employment and child care, control for emu_wage and emu_work
eststo: reg lnscore1_new anyworkall careall_new siminc_both_all_new $reglist1 $reglist2a if score_type==1 & meduc==0
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
*categories, no control function
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 if score_type==1 & meduc==0
*categories, control for emu_wage and emu_work
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a if score_type==1 & meduc==0
test emu_wage emu_work_1
estadd scalar p_ww = r(p)

esttab  using "table1_reading_low.csv",  stats(p_ww r2 N ) order(anyworkall careall_new wdall wdcall siminc_both_all_new ${reglist2a}) ///
 nostar title(Table 1: Estimates of Production Function) ///
	r2 nogap b(%12.3fc) se(%12.3fc) compress obslast replace
	
******************************************
eststo clear
*Table 2: reading score, high education

*employment and child care, no control function
eststo: reg lnscore1_new anyworkall careall_new siminc_both_all_new $reglist1 if score_type==1 & meduc==1
*employment and child care, control for emu_wage and emu_work
eststo: reg lnscore1_new anyworkall careall_new siminc_both_all_new $reglist1 $reglist2a if score_type==1 & meduc==1
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
*categories, no control function
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 if score_type==1 & meduc==1
*categories, control for emu_wage and emu_work
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a if score_type==1 & meduc==1
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
esttab  using "table2_reading_high.csv",  stats(p_ww r2 N ) order(anyworkall careall_new wdall wdcall siminc_both_all_new ${reglist2a}) ///
 nostar title(Table 1: Estimates of Production Function) ///
	r2 nogap b(%12.3fc) se(%12.3fc) compress obslast replace
	
******************************************
eststo clear
*Table 3: reading score, robustness, low and high education

*low education 1: categories, add category "best2all"
eststo: reg lnscore1_new wdall wdcall best2all siminc_both_all_new $reglist1 $reglist2a if score_type==1 & meduc==0
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
*low education 2: categories, control for emu_preg
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2b  if score_type==1 & meduc==0
test emu_wage emu_preg_1
estadd scalar p_wp = r(p)
*low education 3: categories, control for emu_wage and emu_work, control for school
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a i.qt if score_type==1 & meduc==0
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
testparm i.qt
estadd scalar p_qt = r(p)
*low education 4: first child only
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a if score_type==1 & meduc==0 & child2==0
test emu_wage emu_work_1
estadd scalar p_ww = r(p)

esttab  using "table3_reading_robust_low.csv",  stats(p_ww p_wp p_qt r2 N ) order(wdall wdcall best2all siminc_both_all_new ${reglist2a} emu_preg_1 *.qt) ///
 nostar title(Table 1: Estimates of Production Function) ///
	r2 nogap b(%12.3fc) se(%12.3fc) compress obslast replace
	
eststo clear
*high education 1: categories, add category "best2all"
eststo: reg lnscore1_new wdall wdcall best2all siminc_both_all_new $reglist1 $reglist2a if score_type==1 & meduc==1
test emu_wage emu_work_1
estadd scalar p_ww = r(p)

*high education 2: categories, control for emu_preg
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2b  if score_type==1 & meduc==1
test emu_wage emu_preg_1
estadd scalar p_wp = r(p)

*high education 3: categories, control for emu_wage and emu_work, control for school
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a i.qt if score_type==1 & meduc==1
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
testparm i.qt
estadd scalar p_qt = r(p)
*high education 4: first child only
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a if score_type==1 & meduc==1 & child2==0
test emu_wage emu_work_1
estadd scalar p_ww = r(p)

esttab  using "table3_reading_robust_high.csv",  stats(p_ww p_wp p_qt r2 N ) order(wdall wdcall best2all siminc_both_all_new ${reglist2a} emu_preg_1 *.qt) ///
 nostar title(Table 1: Estimates of Production Function) ///
	r2 nogap b(%12.3fc) se(%12.3fc) compress obslast replace
	
******************************************
eststo clear
*Table 4: math and english score, low education

*low 1: categories, no control function
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 if score_type==0 & meduc==0
*low 2: categories, control for emu_wage and emu_work
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a if score_type==0 & meduc==0
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
*low 3: categories, control for emu_wage and emu_work, control for school
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a i.qt if score_type==0 & meduc==0
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
testparm i.qt
estadd scalar p_qt = r(p)
*english score, low education
*low 1: categories, no control function
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 if score_type==2 & meduc==0
*low 2: categories, control for emu_wage and emu_work
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a if score_type==2 & meduc==0
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
*low 3: categories, control for emu_wage and emu_work, control for school
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a i.qt if score_type==2 & meduc==0
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
testparm i.qt
estadd scalar p_qt = r(p)

esttab  using "table4_math_english_low.csv",  stats(p_ww p_qt r2 N ) order(wdall wdcall siminc_both_all_new ${reglist2a} *.qt)  ///
 nostar title(Table 1: Estimates of Production Function) ///
	r2 nogap b(%12.3fc) se(%12.3fc) compress obslast replace
	
******************************************
eststo clear

*Table 5: math and english score, high education

*low 1: categories, no control function
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 if score_type==0 & meduc==1
*low 2: categories, control for emu_wage and emu_work
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a if score_type==0 & meduc==1
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
*low 3: categories, control for emu_wage and emu_work, control for school
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a i.qt if score_type==0 & meduc==1
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
testparm i.qt
estadd scalar p_qt = r(p)
*english score, low education, 8 specifications
*low 1: categories, no control function
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 if score_type==2 & meduc==1
*low 2: categories, control for emu_wage and emu_work
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a if score_type==2 & meduc==1
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
*low 3: categories, control for emu_wage and emu_work, control for school
eststo: reg lnscore1_new wdall wdcall siminc_both_all_new $reglist1 $reglist2a i.qt if score_type==2 & meduc==1
test emu_wage emu_work_1
estadd scalar p_ww = r(p)
testparm i.qt
estadd scalar p_qt = r(p)

esttab  using "table4_math_english_high.csv",  stats(p_ww p_qt r2 N ) order(wdall wdcall siminc_both_all_new ${reglist2a} *.qt)  ///
 nostar title(Table 1: Estimates of Production Function) ///
	r2 nogap b(%12.3fc) se(%12.3fc) compress obslast replace
	
******************************************

log close
