*Codes for "Life-Cycle and Intergenerational Effects of Child Care Reforms" published in Quantitative Economics
* copyright by Marc K. Chan and Kai Liu, 2017;
* simulate changes in cognitive score;
*************************************************************;
%let simpath= directoryhere;
%let simnum = 4;
%let simfilename = simulate4_&simnum..txt;

data sim;
	infile "&simpath.&simfilename";
	input nmpid simno swave type cftype choicenew utility earn obsearn nchildnow workexp lwork lcare lpregnant age1new age2new tlstart benefit pcare pinc;
run;

data motherdata;
	set sim;
	siminc=(obsearn+benefit+pinc);
	if choicenew in (2,3,5,6,8,9,11,12) then work=1;
	else work=0;
	if choicenew in (3,6,9,12) then ft=1;
	else ft=0;
	if work=1 & ft=0 then pt=1;
	else pt=0;
	if choicenew in (4,5,6,10,11,12) & age1new in (1,2,3) then care_1=1;
	else care_1=0;
	if choicenew in (7,8,9,10,11,12) then preg=1;
	else preg=0;
	if swave <=15;
run;
data temp6;
	set motherdata;
	if nchildnow>0;
run;
proc sql; 
	create table temp6_index as
	select p.nmpid, p.simno, count(*) as count, p.type from
	temp6 as p
	group by nmpid, simno, type;
quit;
data temp6_index;
	set temp6_index;
	bwave = 15 - count +1;
run;

proc transpose data=temp6 out=temp7a prefix=ft;
	by nmpid simno;
	var ft;
run;
proc transpose data=temp6 out=temp7b prefix=pt;
	by nmpid simno;
	var pt;
run;
proc transpose data=temp6 out=temp7c prefix=care_1;
	by nmpid simno;
	var care_1;
run;
proc transpose data=temp6 out=temp7d prefix=siminc;
	by nmpid simno;
	var siminc;
run;
data temp7;
	merge temp7a temp7b temp7c temp7d;
run;

data temp8;
	set temp7;
	siminc_sum = (siminc1+siminc2+siminc3+siminc4)*1000;
	if siminc_sum ne . then simincall = log(max(siminc_sum,1));
	work2 = (pt2=1)*1 + (ft2=1)*2;
	work3 = (pt3=1)*1 + (ft3=1)*2;
	work4 = (pt4=1)*1 + (ft4=1)*2;
	wd2 = (work2 ne 0 & care_12=0);
	wd3 = (work3 ne 0 & care_13=0);
	wd4 = (work4 ne 0 & care_14=0);
	wdall = wd2+wd3+wd4;
	wd2c = (work2=0 & care_12=0);
	wd3c = (work3=0 & care_13=0);
	wd4c = (work4=0 & care_14=0);
	wdcall = wd2c + wd3c + wd4c;
	if simincall = . then do;
		wdall = .;
		wdcall = .;
	end;
	keep nmpid simno simincall wdall wdcall;
run;
proc sql;
	create table temp9 as 
	select p.*, l.bwave, l.type
	from temp8 as p, temp6_index as l
	where p.nmpid=l.nmpid & p.simno=l.simno;
quit;
* define baseline scenario child data;
data base&simnum;
	set temp9;
	rename simincall=simincall&simnum wdall=wdall&simnum wdcall=wdcall&simnum bwave=bwave&simnum;
run;



proc sql;
	create table match as
	select p.*, l.*
	from base0 as p full outer join base&simnum  (rename = (type = type&simnum)) as l
	on p.nmpid = l.nmpid & p.simno = l.simno;
quit;
data match;
	set match;
	bstatus = (simincall0 ne . & simincall&simnum ne .)*1 + (simincall0 = . & simincall&simnum ne .)*2 + 
		(simincall0 ne . & simincall&simnum = .)*3 + (simincall0 = . & simincall&simnum = .)*4;
	if type=. then type=type&simnum;
run;
ods html style=minimal;
proc means data=match mean std;
	class bstatus;
	var simincall0 wdall0 wdcall0 simincall&simnum wdall&simnum wdcall&simnum;
	where bstatus in (1,2);
run;
ods html close;

ods html style=minimal;
proc freq data=match;
	table type*bstatus / nofreq norow nopercent;
	where bstatus in (1,2);
run;
ods html close;

