*Codes for "Life-Cycle and Intergenerational Effects of Child Care Reforms" published in Quantitative Economics
* copyright by Marc K. Chan and Kai Liu, 2017
*reads simulation files and output key reults;
%let simpath = directoryhere;
%let filetype = sim;
%let simfilename = simulate4_simfit.txt;
%let simfilename = simulate4.txt;

data sim;
	infile "&simpath.&simfilename";
	input nmpid simno swave type cftype choicenew utility earn obsearn nchildnow workexp lwork lcare lpregnant age1new age2new tlstart benefit careprice;
run;

data &filetype;
	set &filetype;
	if choicenew in (2,3,5,6,8,9,11,12) then work=1;
	else work=0;
	if choicenew in (3,6,9,12) then ft=1;
	else ft=0;
	if work=1 & ft=0 then pt=1;
	else pt=0;
	if choicenew in (4,5,6,10,11,12) then care=1;
	else care=0;
	if choicenew in (7,8,9,10,11,12) then preg=1;
	else preg=0;
	if obsearn=0 then earn_a=.;
	else earn_a=obsearn;

	if work=1 & preg=1 then preg1=1;	*employed and pregnant;
	else preg1=0;
	if work=0 & preg=1 then preg0=1;	*not employed and pregnant;
	else preg0=0;
	if age1new in (1,2,3) or age2new in (1,2,3) then yngchild=1;
	else yngchild=0;
	if age1new in (1,2,3) or age2new in (1,2,3) or lpregnant=1 then yngchild_0=1; * include age 0 children;
	else yngchild_0=0;
	if yngchild=1 then do;
		if care=1 then care_a=1;
		else care_a=0;
		if work=0 & care=0 then yng_1=1;
		else yng_1=0;
		if work=1 & care=0 then yng_2=1;
		else yng_2=0;
		if work=0 & care=1 then yng_3=1;
		else yng_3=0;
		if work=1 & care=1 then yng_4=1;
		else yng_4=0;
	end;
	if yngchild=1 & work=1 then do;
		if care=1 then care_w=1;
		else care_w=0;
	end;
	if preg=1 then do;
		if ft=1 then work_p = 1;
		else work_p = 0;
	end;
run;
data temp;
	set &filetype;
	by nmpid simno;
	if first.simno then do;
		cumutil = utility;
		cumearn = obsearn;
		cumben = benefit;
		cumcareprice = careprice;
	end;
	else do;
		cumutil+utility*0.9**(swave-2);
		cumearn+obsearn*0.9**(swave-2);
		cumben+benefit*0.9**(swave-2);
		cumcareprice+careprice*0.9**(swave-2);
	end;
run;

/* merge with original data set */
data nmpidindex;
	set final_low_v1;
	*set final_high_v1;
	by nmpid;
	if first.nmpid;
	mcohort = year - mage;
	keep nmpid mcohort;
run;

data temp1;
	merge temp nmpidindex;
	by nmpid;
run;
data temp1;	
	set temp1;
	year = mcohort + 20 + swave-2;		/* depends on low high (20 or 24) */
*	year = mcohort + 24 + swave-2;		
	refyear = year-1998;
	if care_a ne . then do;
		care_informal = (care_a=0 & work=1);
		care_maternal = (care_a=0 & work=0);
		care_best	  = (care_a=1 & work=0);
		care_best2	  = (care_a=1 & work=1);
	end;
run;
data temp1a;
	set temp1;
	where swave>=2 & swave<=9;
run;

/* obtain simulation by age/wave */
ods html style=minimal;
proc tabulate data=temp1;
	class swave;
	var work ft pt care care_a care_w preg nchildnow yngchild_0 care_informal care_maternal care_best care_best2 cumearn cumben cumutil;
	table swave='',(work ft pt care care_a care_w preg nchildnow yngchild_0 care_informal care_maternal care_best care_best2 cumearn cumben cumutil)*mean*(f=6.3);
run;
ods html close;

/*Obtain simulation by year */
ods html style=minimal;
proc tabulate data=temp1a;
	class refyear;
	var work ft pt care care_a care_w preg nchildnow yngchild_0 care_informal care_maternal care_best care_best2;
	table refyear='',(work ft pt care care_a care_w preg nchildnow care_informal care_maternal care_best care_best2)*(mean='')*(f=6.3);
run;
ods html close;

/*for counterfactual simulations only*/
ods html style=minimal;
proc tabulate data=temp1;
	class swave;
	var work ft pt care care_a care_w care_informal care_maternal care_best care_best2 preg nchildnow yngchild_0 cumearn cumben cumutil cumcareprice;
	table swave=''*(work pt ft care care_a care_w care_best care_best2 care_maternal care_informal preg yngchild_0 nchildnow cumearn cumben cumutil cumcareprice),(mean='')*(f=6.3);
	where swave in (6,12);
run;
ods html close;
